@@ -1,5 +1,13 @@
-;;; Compiled by f2cl version 2.0 beta Date: 2006/12/21 03:42:11 
-;;; Using Lisp CMU Common Lisp CVS Head 2006-12-02 00:15:46 (19D)
+;;; Compiled by f2cl version:
+;;; ("f2cl1.l,v 2edcbd958861 2012/05/30 03:34:52 toy $"
+;;;  "f2cl2.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl3.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl4.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl5.l,v 3fe93de3be82 2012/05/06 02:17:14 toy $"
+;;;  "f2cl6.l,v 1d5cbacbb977 2008/08/24 00:56:27 rtoy $"
+;;;  "macros.l,v 3fe93de3be82 2012/05/06 02:17:14 toy $")
+
+;;; Using Lisp CMU Common Lisp 20d (20D Unicode)
 ;;; 
 ;;; Options: ((:prune-labels nil) (:auto-save t) (:relaxed-array-decls t)
 ;;;           (:coerce-assigns :as-needed) (:array-type ':array)
@@ -10,12 +18,14 @@
 
 
 (let* ((one (f2cl-lib:cmplx 1.0 0.0)) (zero (f2cl-lib:cmplx 0.0 0.0)))
-  (declare (type (f2cl-lib:complex16) one) (type (f2cl-lib:complex16) zero))
+  (declare (type (f2cl-lib:complex16) one)
+           (type (f2cl-lib:complex16) zero)
+           (ignorable one zero))
   (defun zgemm (transa transb m n k alpha a lda b ldb$ beta c ldc)
     (declare (type (array f2cl-lib:complex16 (*)) c b a)
              (type (f2cl-lib:complex16) beta alpha)
              (type (f2cl-lib:integer4) ldc ldb$ lda k n m)
-             (type (simple-array character (*)) transb transa))
+             (type (simple-string *) transb transa))
     (f2cl-lib:with-multi-array-data
         ((transa character transa-%data% transa-%offset%)
          (transb character transb-%data% transb-%offset%)
@@ -567,8 +577,7 @@
 (eval-when (:load-toplevel :compile-toplevel :execute)
   (setf (gethash 'fortran-to-lisp::zgemm fortran-to-lisp::*f2cl-function-info*)
           (fortran-to-lisp::make-f2cl-finfo
-           :arg-types '((simple-array character (1))
-                        (simple-array character (1))
+           :arg-types '((simple-string) (simple-string)
                         (fortran-to-lisp::integer4) (fortran-to-lisp::integer4)
                         (fortran-to-lisp::integer4)
                         (fortran-to-lisp::complex16)
