@@ -3,10 +3,10 @@
 ;; which are otherwise very voluminous (and this construct
 ;; is accepted by other CL implementations).
 
-#+gcl (defmacro with-compilation-unit (a &rest b) `(progn ,@b))
+#+gcl (unless (macro-function 'with-compilation-unit)
+	       (defmacro with-compilation-unit (a &rest b) `(progn ,@b)))
 
 (with-compilation-unit nil
-    ($load "polynomialp")
     ($load "mring")
     ($load "lu")
     ($load "linalgcholesky")
