@@ -316,10 +316,9 @@ See comments to $adjust_external_format below for a detailed description.
                ef ))))
       ;;
       #+cmucl (progn
-        #+unix (let ((ef (stream-external-format *standard-output*)) ;; input format remains 'default'
-                     (ef2 (stream-external-format *standard-input*)) ) ;; in test-batch input format is UTF-8
+        #+unix (let ((ef (stream-external-format *standard-output*))) ;; input format remains 'default'
           (cond 
-            ((or (eq ef :utf-8) (eq ef2 :utf-8)) enc)
+            ((check-encoding enc) enc)
             (t (is-ignored enc name "to enable the encoding argument") 
                ef )))
         #-unix enc )
@@ -1110,9 +1109,9 @@ constituent, alphanumericp, alphacharp, digitcharp, lowercasep, uppercasep, char
         (apply #'ssubstfirst `(,new ,old ,str ,@args)) )
       (s-pos-error2 "ssubstfirst") ))
 ;;
-(defun ssubstfirst (new old str &optional (test '$sequal) (start 0) (end nil))
+(defun ssubstfirst (new old str &optional (test '$sequal) (start 0) (end nil) (matched? nil))
   (let ((len (length old))
-        (pos (search old str :test test :start2 start :end2 end)) )
+        (pos (if matched? start (search old str :test test :start2 start :end2 end))))
     (if (null pos)
       str
       (concatenate 'string (subseq str 0 pos) new (subseq str (+ pos len))) )))
@@ -1127,15 +1126,12 @@ constituent, alphanumericp, alphacharp, digitcharp, lowercasep, uppercasep, char
       (s-pos-error2 "ssubst") ))
 ;;
 (defun ssubst (new old str &optional (test '$sequal) (start 0) (end nil))
-  (let ((pos (search old str :test test :start2 start :end2 end)))
-    (if (null pos)
-       str
-       (ssubst new
-               old
-               (ssubstfirst new old str test pos end)
-               test
-               (+ pos (length new))
-               (when end (- (+ end (length new)) (length old))) ))))
+  (let ((pos nil) (n (length new)) (o (length old)))
+    (while (setq pos (search old str :test test :start2 start :end2 end))
+      (setq str (ssubstfirst new old str test pos end t)
+	    start (+ pos n)
+	    end (when end (- (+ end n) o))))
+    str))
 
 
 (defun $sremovefirst (seq str &rest args)
