@@ -128,7 +128,7 @@
     (mapl (lambda (q)
             (unless (subsetl (cdr q) (car q))
               (push (car q) solnl)))
-	  (sort tempsolnl #'(lambda (a b) (> (length a) (length b)))))
+	  (stable-sort tempsolnl #'(lambda (a b) (> (length a) (length b)))));FIXME consider a total order function with #'sort
     solnl))
 
 ;;; (SUBSETL L1 S2)
@@ -616,7 +616,13 @@
        (bakalevel (callsolve poly-and-var)
                   (remove (car poly-and-var) lhsl :test #'equal)
                   var)))
-    (t (callsolve (cons (car lhsl) var)))))
+    ;; LHSL contains one polynomial and we try to solve in one variable VAR.
+    ;; CALLSOLVE can miss solutions when the coefficient of the highest order
+    ;; term in VAR contains other variables in *TVARXLIST**.
+    ;; BAKALEVELSOLVE looks for these missed solutions.
+    (t (nconc (callsolve (cons (car lhsl) var))
+	      (bakalevelsolve (car lhsl) var)))))
+
 
 ;; (EVERY-ATOM PRED X)
 ;;
@@ -798,3 +804,110 @@
 	 (exclude l1 (cdr l2)))
 	(t
 	 (cons (car l2) (exclude l1 (cdr l2))))))
+
+;;; (BAKALEVELSOLVE P VAR)
+;;;
+;;; P is a polynomial in VAR.  VAR may not be the main variable.
+;;; We are trying to solve P in terms of the single variable VAR, so
+;;; the other solution variables (in *TVARXLIST*) are treated as constants.
+;;;
+;;; CALLSOLVE can miss solutions when the leading (highest order)
+;;; coefficient C of VAR contains other variables in *TVARXLIST*.
+;;; Apparently C is implicitly assumed to be non-zero.
+;;;
+;;; Seek these missing solutions by:
+;;; - finding N, the highest power of VAR in P
+;;; - separating P into C*VAR^N + D (VAR may not be the main variable in P)
+;;; - look for solutions of form (C=0,D=0)
+;;;
+;;; An example is solving a*b-c*d in terms of [a,b,c,d]
+;;;
+(defun bakalevelsolve (p var)
+  (let (n c d)
+    (cond
+     ;; a bare coefficient isn't an equation
+     ((pcoefp p) nil)
+     ;; N is highest power of VAR in polynomial P.
+     ;; No equation in VAR if N=0.  No solution.
+     ((= (setq n (p-hipow-var p var)) 0) nil)
+    (t
+      ;; express P as C*VAR^N + D
+      (setf (values c d) (p-coef-x p var n))
+      (if (p-allvars c)   ; Does C contain any variables from *TVARXLIST*
+        (algsys `(,c ,d)) ; Try and solve C=0 and D=0 using algsys
+	 nil)))))        ; C is freeof of *TVARXLIST*.  No solution.
+
+;;; De-duplicated list of variables in polynomial P
+(defun p-allvars (p)
+  (unless (pcoefp p) (remove-duplicates (p-allvars1 p))))
+
+;;; List of variables in polynomial P.  May contain duplicates.
+(defun p-allvars1 (p)
+  (unless (pcoefp p)
+    (cons
+     (p-var p)       ; the main variable of poly
+     (loop           ; Recusively extract variables from coefficients.
+      for (nil ci) on (p-terms p) by #'cddr
+      if (not (pcoefp ci)) append (p-allvars1 ci)))))
+
+;;; Maximum degree of variable V in polynomial P
+;;; V may not be the main variable in P
+(defun p-hipow-var (p v)
+  (cond
+    ((pcoefp p) 0)              ; bare coefficient has degree 0
+    ((eq (p-var p) v) (p-le p)) ; V is main variable - return leading exponent
+    (t
+      (loop              ; Recusively search through coefficients of p
+        for (nil c) on (p-terms p) by #'cddr
+        maximize (p-hipow-var c v)))))
+
+;;; (P-COEF-X P X N)
+;;;
+;;; Separate multivariable polynomial P with main variable V into C*x^n + D
+;;; where X may be different to V.  Return (values C D)
+;;; Assumes terms of P are in decending order
+(defun p-coef-x (p x n)
+  (cond
+   ((pcoefp p)                  ; p is a bare coefficient
+     (if (= n 0)
+       (values p (pzero))       ; c = bare coef, d = 0
+       (values (pzero) p)))     ; c = 0,         d = bare coef
+   ((eq (p-var p) x)            ; main variable V in P is X
+     (p-coef-main p n))
+   (t
+     ;; P is polynomial with main variable V other than X
+     ;; iterate over coefficients ai
+     ;; find (ci di) such that ai = ci*x^n + di
+     ;; C = ck*v^k + ... + c0*v^0, a polynomial in v
+     ;; D = dk*v^k + ... + d0*v^0, a polynomial in v
+     (loop
+       with terms = (p-terms p) with v = (p-var p) with ci with di
+       for (i ai) on terms by #'cddr
+       do (setf (values ci di) (p-coef-x ai x n))
+       unless (pzerop ci) append `(,i ,ci) into cterms
+       unless (pzerop di) append `(,i ,di) into dterms
+       finally (return (values (psimp v cterms) (psimp v dterms)))))))
+
+
+;;; (P-COEF-MAIN P N)
+;;;
+;;; Separate multivariable polynomial P with main variable X into C*x^n + D
+;;; Return (values C D)
+;;; Assumes terms of P are in decending order
+(defun p-coef-main (p n)
+  (if (pcoefp p)              ; bare coefficient
+    (if (= n 0)
+      (values p 0)             ; c = bare coef & d = 0
+      (values 0 p))            ; c = 0         & d = bare coef
+    (loop                      ; proper polynomial
+      with L = (p-terms p)     ; list of terms in polynomial
+      with var = (p-var p)     ; main variable
+      for i  = (pop L)         ; power of term
+      for ai = (pop L)         ; coefficient of term x^i
+      while (>= i n)           ; exit via finally if n-th power not present
+      if (> i n)
+        append `(,i ,ai) into terms       ; accumulate higher order terms
+      else                ; have i==n so C=ai.  Construct D and return
+        do (return (values ai (psimp var (append terms L))))
+      while L          ; exit via finally if all coefficients examined
+      finally (return (values 0 p)))))
