@@ -19,8 +19,6 @@
 (loop for (x y) on '(%cot %tan %csc %sin %sec %cos %coth %tanh %csch %sinh %sech %cosh)
    by #'cddr do (putprop x y 'recip) (putprop y x 'recip))
 
-(defun nill () '(nil))
-
 (defmvar $zeta%pi t)
 
 ;; polynomial predicates and other such things
@@ -62,8 +60,8 @@
 
 ;; factorial stuff
 
-(setq $factlim 100000 ; set to a big integer which will work (not -1)
-      makef nil)
+(defmvar $factlim 100000) ; set to a big integer which will work (not -1)
+(defvar makef nil)
 
 (defmfun $genfact (&rest l)
   (cons '(%genfact) l))
@@ -87,22 +85,21 @@
 ;; This is much faster (3-4 times) than the original factorial
 ;; function.
 
-(defun k (n m) 
-  (if (<= n m)
-      n
-      (* (k n (* 2 m))
-	 (k (- n m) (* 2 m)))))
-
 (defun factorial (n)
-  (if (zerop n)
-      1
-      (k n 1)))
+  (labels ((k (n m) 
+	     (if (<= n m)
+		 n
+		 (* (k n (* 2 m))
+		    (k (- n m) (* 2 m))))))
+    (if (zerop n)
+	1
+	(k n 1))))
 
 ;;; Factorial has mirror symmetry
 
 (defprop mfactorial t commutes-with-conjugate)
 
-(defmfun simpfact (x y z)
+(defun simpfact (x y z)
   (oneargcheck x)
   (setq y (simpcheck (cadr x) z))
   (cond ((and (mnump y)
@@ -179,7 +176,7 @@
 
         ;; Double factorial
 
-        ((eq (caar e) '%factorial_double)
+        ((eq (caar e) '%double_factorial)
          (let ((x (makegamma1 (nth 1 e))))
            (mul
              (power
@@ -345,9 +342,9 @@
 		e)))
 	(t (recur-apply #'makegamma1 e))))
 
-(defmfun simpgfact (x vestigial z)
+(defun simpgfact (x vestigial z)
   (declare (ignore vestigial))
-  (if (not (= (length x) 4)) (wna-err '$genfact))
+  (arg-count-check 3 x)
   (setq z (mapcar #'(lambda (q) (simpcheck q z)) (cdr x)))
   (let ((a (car z)) (b (take '($floor) (cadr z))) (c (caddr z)))
     (cond ((and (fixnump a)
@@ -397,10 +394,13 @@ summation when necessary."
 
 ;; These variables should be initialized where they belong.
 
-(setq $wtlevel nil $cflength 1
-      $weightlevels '((mlist)) *trunclist nil $taylordepth 3
-      $maxtaydiff 4 $verbose nil $psexpand nil ps-bmt-disrep t
-      silent-taylor-flag nil)
+(defmvar $cflength 1)
+(defmvar $taylordepth 3)
+(defmvar $maxtaydiff 4)
+(defmvar $verbose nil)
+(defvar *trunclist nil)
+(defvar ps-bmt-disrep t)
+(defvar silent-taylor-flag nil)
 
 (defmacro sum-arg (sum)
   `(cadr ,sum))
@@ -415,14 +415,15 @@ summation when necessary."
   `(cadr (cdddr ,sum)))
 
 (defmspec $sum (l)
+  (arg-count-check 4 l)
   (setq l (cdr l))
-  (if (= (length l) 4)
-      (dosum (car l) (cadr l) (meval (caddr l)) (meval (cadddr l)) t :evaluate-summand t)
-      (wna-err '$sum)))
+  (dosum (car l) (cadr l) (meval (caddr l)) (meval (cadddr l)) t :evaluate-summand t))
+
 
 (defmspec $lsum (l)
+  (arg-count-check 3 l)
   (setq l (cdr l))
-  (or (= (length l) 3) (wna-err '$lsum))
+  ;;(or (= (length l) 3) (wna-err '$lsum))
   (let ((form (car l))
 	(ind (cadr l))
 	(lis (meval (caddr l)))
@@ -437,7 +438,7 @@ summation when necessary."
 	   ans)
 	  (t `((%lsum) ,form ,ind ,lis)))))
     
-(defmfun simpsum (x y z)
+(defun simpsum (x y z)
   (let (($ratsimpexpons t))
     (setq y (simplifya (sum-arg x) z)))
   (simpsum1 y (sum-index x) (simplifya (sum-lower x) z)
@@ -513,12 +514,10 @@ summation when necessary."
 
 (defun subst-if-not-freeof (x y expr)
   (if ($freeof y expr)
+      ;; suppressing substitution here avoids substituting for
+      ;; local variables recognize by freeof, e.g., formal argument of lambda.
       expr
-      (if (atom expr)
-	  x
-	  (let* ((args (cdr expr))
-		 (L (eval `(mapcar (lambda (a) (subst-if-not-freeof ',x ',y a)) ',args))))
-	    (cons (car expr) L)))))
+      (let ($simp) (maxima-substitute x y expr))))
 
 (defun mevalsumarg (expr ind low hi)
   (if (let (($prederror nil))
@@ -566,130 +565,115 @@ summation when necessary."
             ind (meval (caddr l)) (meval (cadddr l))))))
 
 (defun simpsum1 (e k lo hi)
-  (let ((fact1) (fact2) (acc 0) (n) (sgn) ($prederror nil) (i (gensym)) (ex))
-    (setq lo ($ratdisrep lo))
-    (setq hi ($ratdisrep hi))
-   
-    (setq n ($limit (add 1 (sub hi lo))))
-    (setq sgn ($sign n))
-  
-    (setq fact1 `((mgeqp) ,i ,lo))
-    (setq fact2 `((mgeqp) ,hi ,i))
-    
-    (if (not (eq t (csign lo))) (mfuncall '$assume fact1))
-    (if (not (eq t (csign hi))) (mfuncall '$assume fact2))
-    
-    (setq ex (subst i k e))
-    (setq ex (subst i k ex))
+  (with-new-context (context)
+    (let ((acc 0) (n) (sgn) ($prederror nil) (i (gensym)) (ex))
+      (setq lo ($ratdisrep lo))
+      (setq hi ($ratdisrep hi))
 
-    (setq acc
-	  (cond ((and (eq n '$inf) ($freeof i ex))
-		 (setq sgn (csign ex))
-		 (cond ((eq sgn '$pos) '$inf)
-		       ((eq sgn '$neg) '$minf)
-		       ((eq sgn '$zero) 0)
-		       (t `((%sum simp) ,ex ,i ,lo ,hi))))
+      (setq n ($limit (add 1 (sub hi lo))))
+      (setq sgn ($sign n))
 
-		((and (mbagp e) $listarith)
-		 (simplifya
-		  `((,(caar e)) ,@(mapcar #'(lambda (s) (mfuncall '$sum s k lo hi)) (margs e))) t))
-		 		
-		((or (eq sgn '$neg) (eq sgn '$zero) (eq sgn '$nz)) 0)
-		
-		((like ex 0) 0)
-			
-		(($freeof i ex) (mult n ex))
-			
-		((and (integerp n) (eq sgn '$pos) $simpsum)
-		 (unwind-protect 
-		      (dotimes (j n acc)
-			(setq acc (add acc (resimplify (subst (add j lo) i ex)))))
-		   (mfuncall '$forget fact1)
-		   (mfuncall '$forget fact2)))
-		(t 
-		 (setq ex (subst '%sum '$sum ex))
-		 `((%sum simp) ,(subst k i ex) ,k ,lo ,hi))))
-	      
-    (setq acc (subst k i acc))
-    
-    ;; If expression is still a summation,
-    ;; punt to previous simplification code.
+      (if (not (eq t (csign lo))) (mfuncall '$assume `((mgeqp) ,i ,lo)))
+      (if (not (eq t (csign hi))) (mfuncall '$assume `((mgeqp) ,hi ,i)))
 
-    (if (and $simpsum (op-equalp acc '$sum '%sum))
-	(let* ((args (cdr acc)) (e (first args)) (i (second args)) (i0 (third args)) (i1 (fourth args)))
-	  (setq acc (simpsum1-save e i i0 i1))))
+      (setq ex (subst i k e))
+      (setq ex (subst i k ex))
 
-    (mfuncall '$forget fact1)
-    (mfuncall '$forget fact2)
+      (setq acc
+            (cond ((and (eq n '$inf) ($freeof i ex))
+                   (setq sgn (csign ex))
+                   (cond ((eq sgn '$pos) '$inf)
+                         ((eq sgn '$neg) '$minf)
+                         ((eq sgn '$zero) 0)
+                         (t `((%sum simp) ,ex ,i ,lo ,hi))))
 
-    acc))
+                  ((and (mbagp e) $listarith)
+                   (simplifya
+                    `((,(caar e)) ,@(mapcar #'(lambda (s) (mfuncall '$sum s k lo hi)) (margs e))) t))
+
+                  ((or (eq sgn '$neg) (eq sgn '$zero) (eq sgn '$nz)) 0)
+
+                  ((like ex 0) 0)
+
+                  (($freeof i ex) (mult n ex))
+
+                  ((and (integerp n) (eq sgn '$pos) $simpsum)
+                   (dotimes (j n acc)
+                     (setq acc (add acc (resimplify (subst (add j lo) i ex))))))
+
+                  (t
+                   (setq ex (subst '%sum '$sum ex))
+                   `((%sum simp) ,(subst k i ex) ,k ,lo ,hi))))
+
+      (setq acc (subst k i acc))
+
+      ;; If expression is still a summation,
+      ;; punt to previous simplification code.
+
+      (if (and $simpsum (op-equalp acc '$sum '%sum))
+        (let* ((args (cdr acc)) (e (first args)) (i (second args)) (i0 (third args)) (i1 (fourth args)))
+          (setq acc (simpsum1-save e i i0 i1))))
+
+      acc)))
 
 (defun simpprod1 (e k lo hi)
-  (let ((fact1) (fact2) (acc 1) (n) (sgn) ($prederror nil) (i (gensym)) (ex) (ex-mag) (realp))
+  (with-new-context (context)
+    (let ((acc 1) (n) (sgn) ($prederror nil) (i (gensym)) (ex) (ex-mag) (realp))
 
-    (setq lo ($ratdisrep lo))
-    (setq hi ($ratdisrep hi))
-    (setq n ($limit (add 1 (sub hi lo))))
-    (setq sgn ($sign n))
-    (setq fact1 `((mgeqp) ,i ,lo))
-    (setq fact2 `((mgeqp) ,hi ,i))
-    
-    (if (not (eq t (csign lo))) (mfuncall '$assume fact1))
-    (if (not (eq t (csign hi))) (mfuncall '$assume fact2))
+      (setq lo ($ratdisrep lo))
+      (setq hi ($ratdisrep hi))
+      (setq n ($limit (add 1 (sub hi lo))))
+      (setq sgn ($sign n))
 
-    (setq ex (subst i k e))
-    (setq ex (subst i k ex))
+      (if (not (eq t (csign lo))) (mfuncall '$assume `((mgeqp) ,i ,lo)))
+      (if (not (eq t (csign hi))) (mfuncall '$assume `((mgeqp) ,hi ,i)))
 
-    (setq acc
-          (cond
-            ((like ex 1) 1)
+      (setq ex (subst i k e))
+      (setq ex (subst i k ex))
 
-            ((and (eq n '$inf) ($freeof i ex))
-             (setq ex-mag (mfuncall '$cabs ex))
-             (setq realp (mfuncall '$imagpart ex))
-             (setq realp (mevalp `((mequal) 0 ,realp)))
-             
-             (cond ((eq t (mevalp `((mlessp) ,ex-mag 1))) 0)
-                   ((and (eq realp t) (eq t (mevalp `((mgreaterp) ,ex 1)))) '$inf)
-                   ((eq t (mevalp `((mgreaterp) ,ex-mag 1))) '$infinity)
-                   ((eq t (mevalp `((mequal) 1 ,ex-mag))) '$und)
-                   (t `((%product) ,e ,k ,lo ,hi))))
-            
-            ((or (eq sgn '$neg) (eq sgn '$zero) (eq sgn '$nz))
-             1)
+      (setq acc
+            (cond
+              ((like ex 1) 1)
 
-            ((and (mbagp e) $listarith)
-             (simplifya
-	      `((,(caar e)) ,@(mapcar #'(lambda (s) (mfuncall '$product s k lo hi)) (margs e))) t))
-            
-            (($freeof i ex) (power ex n))
-            
-            ((and (integerp n) (eq sgn '$pos) $simpproduct)
-             (unwind-protect
-		  (dotimes (j n acc)
-		    (setq acc (mult acc (resimplify (subst (add j lo) i ex)))))
-               
-               (mfuncall '$forget fact1)
-               (mfuncall '$forget fact2)))
-            
-            (t
-	     (setq ex (subst '%product '$product ex))
-	     `((%product simp) ,(subst k i ex) ,k ,lo ,hi))))
+              ((and (eq n '$inf) ($freeof i ex))
+               (setq ex-mag (mfuncall '$cabs ex))
+               (setq realp (mfuncall '$imagpart ex))
+               (setq realp (mevalp `((mequal) 0 ,realp)))
 
-    ;; Hmm, this is curious... don't call existing product simplifications
-    ;; if index range is infinite -- what's up with that??
+               (cond ((eq t (mevalp `((mlessp) ,ex-mag 1))) 0)
+                     ((and (eq realp t) (eq t (mevalp `((mgreaterp) ,ex 1)))) '$inf)
+                     ((eq t (mevalp `((mgreaterp) ,ex-mag 1))) '$infinity)
+                     ((eq t (mevalp `((mequal) 1 ,ex-mag))) '$und)
+                     (t `((%product) ,e ,k ,lo ,hi))))
 
-    (if (and $simpproduct (op-equalp acc '$product '%product) (not (like n '$inf)))
-	(let* ((args (cdr acc)) (e (first args)) (i (second args)) (i0 (third args)) (i1 (fourth args)))
-	  (setq acc (simpprod1-save e i i0 i1))))
+              ((or (eq sgn '$neg) (eq sgn '$zero) (eq sgn '$nz))
+               1)
 
-    (setq acc (subst k i acc))
-    (setq acc (subst '%product '$product acc))
+              ((and (mbagp e) $listarith)
+               (simplifya
+                `((,(caar e)) ,@(mapcar #'(lambda (s) (mfuncall '$product s k lo hi)) (margs e))) t))
 
-    (mfuncall '$forget fact1)
-    (mfuncall '$forget fact2)
+              (($freeof i ex) (power ex n))
 
-    acc))
+              ((and (integerp n) (eq sgn '$pos) $simpproduct)
+               (dotimes (j n acc)
+                 (setq acc (mult acc (resimplify (subst (add j lo) i ex))))))
+
+              (t
+               (setq ex (subst '%product '$product ex))
+               `((%product simp) ,(subst k i ex) ,k ,lo ,hi))))
+
+      ;; Hmm, this is curious... don't call existing product simplifications
+      ;; if index range is infinite -- what's up with that??
+
+      (if (and $simpproduct (op-equalp acc '$product '%product) (not (like n '$inf)))
+        (let* ((args (cdr acc)) (e (first args)) (i (second args)) (i0 (third args)) (i1 (fourth args)))
+          (setq acc (simpprod1-save e i i0 i1))))
+
+      (setq acc (subst k i acc))
+      (setq acc (subst '%product '$product acc))
+
+      acc)))
 
 ; This function was SIMPPROD1 until the sum/product code was revised Nov 2005.
 ; The revised code punts back to this function since this code knows
@@ -717,7 +701,7 @@ summation when necessary."
 				     (list '(mabs) lo))
 			       (cond ((member (asksign hi) '($zero $positive) :test #'eq)
 				      0)
-				     (t (prog2 0
+				     (t (prog1
 					    (m^ -1 (m+ hi lo 1))
 					  (setq hi (list '(mabs) hi)))))
 			       (list '(mfactorial) hi))))))))
@@ -727,9 +711,7 @@ summation when necessary."
 ;; multiplication of sums
 
 (defun gensumindex ()
-  (implode (nconc (exploden $genindex)
-		  (and $gensumnum
-		       (mexploden (setq $gensumnum (1+ $gensumnum)))))))
+  (intern (format nil "~S~D" $genindex (incf $gensumnum))))
 
 (defun sumtimes (x y)
   (cond ((null x) y)
@@ -819,6 +801,36 @@ summation when necessary."
 	(t (let ((opers-list (cdr opers-list)))
 	     (oper-apply e z)))))
 
+;; Define an operator simplification, the same as antisymmetric, commutative, linear, etc.
+;; Here OP = operator name, FN = function of 1 argument to carry out operator-specific simplification.
+;; 1. push operator name onto OPERS
+;; 2. update $OPPROPERTIES
+;; 3. push operator name and glue code onto *OPERS-LIST
+;; 4. declare operator name as a feature, so declare(..., <op>) is recognized
+
+(defmfun $define_opproperty (op fn)
+  (unless (symbolp op)
+    (merror "define_opproperty: first argument must be a symbol; found: ~M" op))
+  (unless (or (symbolp fn) (and (consp fn) (eq (caar fn) 'lambda)))
+    (merror "define_opproperty: second argument must be a symbol or lambda expression; found: ~M" fn))
+  (push op opers)
+  (setq $opproperties (cons '(mlist simp) (reverse opers)))
+  (let
+    ((fn-glue (coerce (if (symbolp fn)
+                        `(lambda (e z)
+                           (declare (ignorable z))
+                           (if (or (fboundp ',fn) (mget ',fn 'mexpr))
+                             (let ((e1 (let ($simp) (mfuncall ',fn e))))
+                               (if ($mapatom e1) e1 (oper-apply e1 nil)))
+                             (list '(,fn) (let ((*opers-list (cdr *opers-list))) (oper-apply e z)))))
+                        `(lambda (e z)
+                           (declare (ignore z))
+                           (let ((e1 (let ($simp) (mfuncall ',fn e))))
+                             (if ($mapatom e1) e1 (oper-apply e1 nil)))))
+                      'function)))
+    (push `(,op . ,fn-glue) *opers-list))
+  (mfuncall '$declare op '$feature))
+
 (defun linearize1 (e z)		; z = t means args already simplified.
   (linearize2 (cons (car e) (mapcar #'(lambda (q) (simpcheck q z)) (cdr e)))
 	      nil))
@@ -834,7 +846,7 @@ summation when necessary."
       ((null l)
        (cons (simplifya (cons (list (caar e)) a) nil)
 	     (simplifya (cons (list (caar e)) b) nil)))
-    (if (or (mnump (car l)) (constant (car l)))
+    (if ($constantp (car l))
 	(setq a (cons (car l) a))
 	(setq b (cons (car l) b)))))
 
@@ -967,13 +979,13 @@ summation when necessary."
 (setq opers (cons '$oddfun opers)
       *opers-list (cons '($oddfun . oddfun) *opers-list))
 
-(defmfun evenfun (e z)
+(defun evenfun (e z)
   (if (or (null (cdr e)) (cddr e))
       (merror (intl:gettext "Function declared 'even' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
     (oper-apply (list (car e) (if (mminusp arg) (neg arg) arg)) t)))
 
-(defmfun oddfun (e z)
+(defun oddfun (e z)
   (if (or (null (cdr e)) (cddr e))
       (merror (intl:gettext "Function declared 'odd' takes exactly one argument; found ~M") e))
   (let ((arg (simpcheck (cadr e) z)))
@@ -986,7 +998,7 @@ summation when necessary."
 (setq opers (cons '$symmetric opers)
       *opers-list (cons '($symmetric . commutative1) *opers-list))
 
-(defmfun commutative1 (e z)
+(defun commutative1 (e z)
   (oper-apply (cons (car e)
 		    (reverse
 		     (sort (mapcar #'(lambda (q) (simpcheck q z))
@@ -997,42 +1009,51 @@ summation when necessary."
 (setq opers (cons '$antisymmetric opers)
       *opers-list (cons '($antisymmetric . antisym) *opers-list))
 
-(declare-top (special sign))
+(defun antisym (e z)
+  (when (and $dotscrules (mnctimesp e))
+    (let ($dotexptsimp)
+      (setq e (simpnct e 1 nil))))
+  (if ($atom e) e (antisym1 e z)))
 
-(defmfun antisym (e z)
-  (let ((l (mapcar #'(lambda (q) (simpcheck q z)) (cdr e))))
-    (let (sign) (if (or (not (eq (caar e) 'mnctimes)) (freel l 'mnctimes))
-		    (setq l (bbsort1 l)))
-	 (cond ((equal l 0) 0)
-	       ((prog1 (null sign) (setq e (oper-apply (cons (car e) l) t)))
-		e)
-	       (t (neg e))))))
+(defun antisym1 (e z)
+  (let ((antisym-sign nil)
+	(l (mapcar #'(lambda (q) (simpcheck q z)) (cdr e))))
+    (when (or (not (eq (caar e) 'mnctimes)) (freel l 'mnctimes))
+      (multiple-value-setq (l antisym-sign) (bbsort1 l)))
+    (cond ((equal l 0) 0)
+	  ((prog1
+	       (null antisym-sign)
+	     (setq e (oper-apply (cons (car e) l) t)))
+	   e)
+	  (t (neg e)))))
 
 (defun bbsort1 (l)
-  (prog (sl sl1)
-     (if (or (null l) (null (cdr l))) (return l))
-     (setq sign nil sl (list nil (car l)))
+  (prog (sl sl1 antisym-sign)
+     (if (or (null l) (null (cdr l))) (return (values l antisym-sign))
+	 (setq sl (list nil (car l))))
      loop (setq l (cdr l))
-     (if (null l) (return (nreverse (cdr sl))))
+     (if (null l) (return (values (nreverse (cdr sl)) antisym-sign)))
      (setq sl1 sl)
      loop1(cond ((null (cdr sl1)) (rplacd sl1 (cons (car l) nil)))
-		((alike1 (car l) (cadr sl1)) (return 0))
+		((alike1 (car l) (cadr sl1)) (return (values 0 nil)))
 		((great (car l) (cadr sl1)) (rplacd sl1 (cons (car l) (cdr sl1))))
-		(t (setq sign (not sign) sl1 (cdr sl1)) (go loop1)))
+		(t (setq antisym-sign (not antisym-sign) sl1 (cdr sl1)) (go loop1)))
      (go loop)))
 
 (setq opers (cons '$nary opers)
       *opers-list (cons '($nary . nary1) *opers-list))
 
 (defun nary1 (e z)
+  (oper-apply (nary2 e z) z))
+
+(defun nary2 (e z)
   (do
     ((l (cdr e) (cdr l)) (ans) (some-change))
 
     ((null l)
      (if some-change
-       (nary1 (cons (car e) (nreverse ans)) z)
-       (let ((w (get (caar e) 'operators)))
-         (if w (funcall w e 1 z) (simpargs e z)))))
+       (nary2 (cons (car e) (nreverse ans)) z)
+       (simpargs e z)))
 
     (setq
       ans (if (and (not (atom (car l))) (eq (caaar l) (caar e)))
@@ -1044,10 +1065,10 @@ summation when necessary."
 (setq opers (cons '$lassociative opers)
       *opers-list (cons '($lassociative . lassociative) *opers-list))
 
-(defmfun lassociative (e z)
+(defun lassociative (e z)
   (let*
     ((ans0 (oper-apply (cons (car e) (total-nary e)) z))
-     (ans (cdr ans0)))
+     (ans (if (consp ans0) (cdr ans0))))
     (cond ((or (null (cddr ans)) (not (eq (caar ans0) (caar e)))) ans0)
           ((do ((newans (list (car e) (car ans) (cadr ans))
                         (list (car e) newans (car ans)))
@@ -1057,10 +1078,10 @@ summation when necessary."
 (setq opers (cons '$rassociative opers)
       *opers-list (cons '($rassociative . rassociative) *opers-list))
 
-(defmfun rassociative (e z)
+(defun rassociative (e z)
   (let*
     ((ans0 (oper-apply (cons (car e) (total-nary e)) z))
-     (ans (cdr ans0)))
+     (ans (if (consp ans0) (cdr ans0))))
     (cond ((or (null (cddr ans)) (not (eq (caar ans0) (caar e)))) ans0)
 	  (t (setq ans (nreverse ans))
 	     (do ((newans (list (car e) (cadr ans) (car ans))
@@ -1068,7 +1089,7 @@ summation when necessary."
 		  (ans (cddr ans) (cdr ans)))
 		 ((null ans) newans))))))
 
-(defmfun total-nary (e)
+(defun total-nary (e)
   (do ((l (cdr e) (cdr l)) (ans))
       ((null l) (nreverse ans))
     (setq ans (if (and (not (atom (car l))) (eq (caaar l) (caar e)))
