@@ -15,7 +15,7 @@
 
 (declare-top (special $props $dotdistrib))
 
-(defmfun diffint (e x)
+(defun diffint (e x)
   (let (a)
     (cond ((null (cdddr e))
 	   (cond ((alike1 x (caddr e)) (cadr e))
@@ -39,7 +39,7 @@
     (list (if (pzerop u) 0 (mul2 u (maxima-substitute (cadddr e) y (car e))))
 	  (if (pzerop v) 0 (mul3 v (maxima-substitute (caddr e) y (car e)) -1)))))
 
-(defmfun diffsumprod (e x)
+(defun diffsumprod (e x)
   (cond ((or (not ($mapatom x)) (not (free (cadddr e) x)) (not (free (car (cddddr e)) x)))
 	 (diff%deriv (list e x 1)))
 	((eq (caddr e) x) 0)
@@ -50,24 +50,24 @@
 				t))
 	     (if (eq (caar e) '%sum) u (mul2 e u))))))
 
-(defmfun difflaplace (e x)
+(defun difflaplace (e x)
   (cond ((or (not (atom x)) (eq (cadddr e) x)) (diff%deriv (list e x 1)))
 	((eq (caddr e) x) 0)
 	(t ($laplace (sdiff (cadr e) x) (caddr e) (cadddr e)))))
 
-(defmfun diff-%at (e x)
+(defun diff-%at (e x)
   (cond ((freeof x e) 0)
 	((not (freeofl x (hand-side (caddr e) 'r))) (diff%deriv (list e x 1)))
 	(t ($at (sdiff (cadr e) x) (caddr e)))))
 
-(defmfun diffncexpt (e x)
+(defun diffncexpt (e x)
   (let ((base* (cadr e))
 	(pow (caddr e)))
-    (cond ((and (mnump pow) (or (not (eq (ml-typep pow) 'fixnum)) (< pow 0))) ; POW cannot be 0
+    (cond ((and (mnump pow) (or (not (fixnump pow)) (< pow 0))) ; POW cannot be 0
 	   (diff%deriv (list e x 1)))
 	  ((and (atom base*) (eq base* x) (free pow base*))
 	   (mul2* pow (list '(mncexpt) base* (add2 pow -1))))
-	  ((ml-typep pow 'fixnum)
+	  ((fixnump pow)
 	   (let ((deriv (sdiff base* x))
 		 (ans nil))
 	     (do ((i 0 (1+ i))) ((= i pow))
@@ -88,7 +88,7 @@
 		    index 0 (list '(mplus) pow -1)) nil)))
 	  (t (diff%deriv (list e x 1))))))
 
-(defmfun stotaldiff (e)
+(defun stotaldiff (e)
   (cond ((or (mnump e) (constant e)) 0)
 	((or (atom e) (member 'array (cdar e) :test #'eq))
 	 (let ((w (mget (if (atom e) e (caar e)) 'depends)))
@@ -103,7 +103,7 @@
 	   (add2 (ncmuln (cons (stotaldiff (cadr e)) (cddr e)) t)
 		 (ncmul2 (cadr e) (stotaldiff (ncmuln (cddr e) t))))))
 	((eq (caar e) 'mncexpt)
-	 (if (and (ml-typep (caddr e) 'fixnum) (> (caddr e) 0))
+	 (if (and (fixnump (caddr e)) (> (caddr e) 0))
 	     (stotaldiff (list '(mnctimes) (cadr e)
 			       (ncpower (cadr e) (1- (caddr e)))))
 	     (list '(%derivative) e)))
@@ -131,7 +131,7 @@
 ;;;; AT
 
 ;;dummy-variable-operators is defined in COMM, which uses it inside of SUBST1.
-(declare-top (special atvars ateqs *atp* munbound dummy-variable-operators))
+(declare-top (special atvars *atp* munbound dummy-variable-operators))
 
 (defmfun $atvalue (exp eqs val)
   (let (dl vl fun)
@@ -139,7 +139,9 @@
 	  ((or (atom exp) (and (eq (caar exp) '%derivative) (atom (cadr exp))))
 	   (improper-arg-err exp '$atvalue)))
     (cond ((not (eq (caar exp) '%derivative))
-	   (setq fun (caar exp) vl (cdr exp) dl (listof0s vl)))
+	   (setq fun (caar exp)
+                 vl (cdr exp)
+                 dl (make-list (length vl) :initial-element 0)))
 	  (t (setq fun (caaadr exp) vl (cdadr exp))
 	     (dolist (v vl)
 	       (setq dl (nconc dl (ncons (or (getf (cddr exp) v) 0)))))))
@@ -170,19 +172,50 @@
 (defun simp-%at (expr ignored simp-flag)
   (declare (ignore ignored))
   (twoargcheck expr)
-  (let ((arg (simpcheck (cadr expr) simp-flag))
-        (eqn (if ($listp (caddr expr))
-                 (cons '(mlist simp) (cdr ($sort (caddr expr))))
-                 (caddr expr))))
+  (let* ((arg (simpcheck (cadr expr) simp-flag))
+         (e (resimplify (caddr expr)))
+         (eqn (if ($listp e)
+                  (if (= ($length e) 1) ($first e) (cons '(mlist simp) (cdr ($sort e))))
+                  e)))
     (cond (($constantp arg) arg)
           ((alike1 eqn '((mlist))) arg)
+          ((at-not-dependent eqn arg))
           (t (eqtest (list '(%at) arg eqn) expr)))))
 
+;; Remove any variable from EQN if ARG is not dependent on it.
+(defun at-not-dependent (eqn arg)
+  (if (eq (caar eqn) 'mequal)
+    (setq eqn (list '(mlist) eqn)))
+  (multiple-value-bind (e0 e1) (at-not-dependent-find-vars eqn arg)
+    (if e0
+      (if e1
+        (let*
+          ((e1 (mapcar #'(lambda (x) (list '(mequal) x ($assoc x eqn))) e1))
+           (eqn1 (if (= (length e1) 1) (first e1) (cons '(mlist) e1))))
+          (list '(%at) arg eqn1))
+        arg))))
+
+;; Test dependence via derivative to account for declared dependencies.
+(defun at-not-dependent-find-vars (eqn arg)
+  (let ((e (mapcar #'second (rest eqn))))
+    (partition-by #'(lambda (x) (at-not-dependent-find-vars-1 x arg)) e)))
+
+(defun at-not-dependent-find-vars-1 (x arg)
+  (if ($mapatom x)
+    (eql (mfuncall '$diff arg x) 0)
+    ;; We might be called with something like -1*x as the variable.
+    ;; (That might or might not be a bug in itself, but let it go for the moment.)
+    ;; Try to extract a variable and test for dependence on that.
+    ;; If there are 2 or more variables, return NIL (i.e., not at-not-dependent).
+    (let ((v ($listofvars x)))
+      (if (eql ($length v) 1)
+        (at-not-dependent-find-vars-1 ($first v) arg)))))
+
 (defmfun $at (expr ateqs)
   (if (notloreq ateqs) (improper-arg-err ateqs '$at))
-  (atscan (let ((*atp* t)) ($psubstitute ateqs expr))))
+  (atscan (let ((*atp* t)) ($psubstitute ateqs expr)) ateqs))
 
-(defun atscan (expr)
+(defun atscan (expr ateqs)
   (cond ((or (atom expr)
              (eq (caar expr) 'mrat)
              (like ateqs '((mlist))))
@@ -199,10 +232,10 @@
         ((member (caar expr) dummy-variable-operators :test #'eq)
          (list '(%at) expr ateqs))
         ((at1 expr))
-        (t (recur-apply #'atscan expr))))
+        (t (recur-apply #'(lambda (x) (atscan x ateqs)) expr))))
 
 (defun at1 (expr)
-  (atfind (caar expr) (cdr expr) (listof0s (cdr expr))))
+  (atfind (caar expr) (cdr expr) (make-list (length (cdr expr)) :initial-element 0)))
 
 (defun atfind (fun vl dl)
   (do ((atvalues (mget fun 'atvalues) (cdr atvalues)))
@@ -217,58 +250,80 @@
                     (atvarschk vl)
                     (substitutel vl atvars (caddar atvalues)))))))
 
-(defun listof0s (llist)
-  (do ((llist llist (cdr llist)) (l nil (cons 0 l)))
-      ((null llist) l)))
-
-(declare-top (special $ratfac genvar varlist $keepfloat *e*))
+(declare-top (special $ratfac genvar varlist $keepfloat))
 
 (defmvar $logconcoeffp nil)
-(defmvar superlogcon t)
-(defmvar $superlogcon t)
 
 (defmfun $logcontract (e)
-  (lgccheck (logcon e))) ; E is assumed to be simplified.
+  (lgcreciprocal (logcon e))) ; E is assumed to be simplified.
 
 (defun logcon (e)
   (cond ((atom e) e)
 	((member (caar e) '(mplus mtimes) :test #'eq)
-	 (if (and $superlogcon (not (lgcsimplep e))) (setq e (lgcsort e)))
+	 (if (not (lgcsimplep e)) (setq e (lgcsort e)))
          (cond ((mplusp e) (lgcplus e))
                ((mtimesp e) (lgctimes e))
                (t (logcon e))))
 	(t (recur-apply #'logcon e))))
 
+;; The logcontract algorithm for a sum.
+;;
+;; The function accumulates the arguments of things like log(a)+log(b) into a
+;; list called LOG. It calls out to lgctimes to deal with things like
+;; a*log(b). When all the arguments have been processed, it simplifies all the
+;; logarithmic arguments using sratsimp.
 (defun lgcplus (e)
-  (do ((x (cdr e) (cdr x)) (log) (notlogs) (y))
-      ((null x)
-       (cond ((null log) (subst0 (cons '(mplus) (nreverse notlogs)) e))
-             (t
-              (setq log (let (($ratfac t)) (sratsimp (muln log t))))
-              (addn (cons (lgcsimp log) notlogs) t))))
-    (cond ((atom (car x)) (setq notlogs (cons (car x) notlogs)))
-	  ((eq (caaar x) '%log) (setq log (cons (logcon (cadar x)) log)))
-	  ((eq (caaar x) 'mtimes)
-	   (setq y (lgctimes (car x)))
-	   (cond ((or (atom y) (not (eq (caar y) '%log)))
-		  (setq notlogs (cons y notlogs)))
-		 (t (setq log (cons (cadr y) log)))))
-	  (t (setq notlogs (cons (logcon (car x)) notlogs))))))
+  (let ((log) (notlogs))
+    (dolist (arg (cdr e))
+      (cond
+        ((atom arg) (push arg notlogs))
+        ;; Only gather up log(x), not log[x]. It's not particularly obvious
+        ;; whether log(x)+log[y] should become log(x*y) or log[x*y], so we just
+        ;; ignore the fact that log[x] is a logarithm.
+        ((and (eq (caar arg) '%log)
+              (not (member 'array (car arg))))
+         (push (logcon (second arg)) log))
+        ((eq (caar arg) 'mtimes)
+         (let ((y (lgctimes arg)))
+           (if (or (atom y) (not (eq (caar y) '%log)))
+               (push y notlogs)
+               (push (cadr y) log))))
+        (t
+         (push (logcon arg) notlogs))))
+    (cond
+      ((null log)
+       (subst0 (cons '(mplus) (nreverse notlogs)) e))
+      (t
+       (let ((simplified-log (lgcsimp
+                              (let (($ratfac t))
+                                (sratsimp (muln log t))))))
+         (addn (cons simplified-log notlogs) t))))))
 
+;; The logcontract algorithm for a product
+;;
+;; The main transformation this does is of the form 3*log(x) => log(x^3). To
+;; make this work, we find the first %log term and insert any coefficients we
+;; find into that. Coefficients are identified by LOGCONCOEFFP, which checks the
+;; $LOGCONCOEFFP user variable.
 (defun lgctimes (e)
+  ;; Apply logcontract to the arguments. It's possible that the subsequent
+  ;; simplification means that the result isn't a product any more. In that
+  ;; case, just return it.
   (setq e (subst0 (cons '(mtimes) (mapcar 'logcon (cdr e))) e))
-  (cond ((not (mtimesp e)) e)
-	(t (do ((x (cdr e) (cdr x)) (log) (notlogs) (decints))
-	       ((null x)
-		(cond ((or (null log) (null decints)) e)
-		      (t (muln (cons (lgcsimp (power log (muln decints t)))
-				     notlogs)
-			       t))))
-	     (cond ((and (null log) (not (atom (car x)))
-			 (eq (caaar x) '%log) (not (equal (cadar x) -1)))
-		    (setq log (cadar x)))
-		   ((logconcoeffp (car x)) (setq decints (cons (car x) decints)))
-		   (t (setq notlogs (cons (car x) notlogs))))))))
+  (if (not (mtimesp e))
+      e
+      (let ((log) (notlogs) (decints))
+        (dolist (arg (cdr e))
+          (cond ((and (null log) (not (atom arg))
+                      (eq (caar arg) '%log) (not (equal (cadr arg) -1)))
+                 (setq log (cadr arg)))
+                ((logconcoeffp arg) (push arg decints))
+                (t (setq notlogs (push arg notlogs)))))
+        (cond
+          ((or (null log) (null decints)) e)
+          (t (muln (cons (lgcsimp (power log (muln decints t)))
+                         notlogs)
+                   t))))))
 
 (defun lgcsimp (e)
   (cond ((atom e)
@@ -280,92 +335,90 @@
         (t
          (list '(%log simp) e))))
 
+;; Tests that its argument is a sum of terms that are "simple".
+;;
+;; A "simple" term is either completely free of logarithms, is a logarithm
+;; itself, or is a number times a logarithm.
+;;
+;; This function assumes that its argument is not an atom.
 (defun lgcsimplep (e)
-  (and (eq (caar e) 'mplus)
-       (not (do ((l (cdr e) (cdr l))) ((null l))
-	      (cond ((not (or (atom (car l))
-			      (not (isinop (car l) '%log))
-			      (eq (caaar l) '%log)
-			      (and (eq (caaar l) 'mtimes)
-				   (null (cdddar l))
-				   (mnump (cadar l))
-				   (not (atom (caddar l)))
-				   (eq (caar (caddar l)) '%log))))
-		     (return t)))))))
+  (flet ((lgc-nonsimple-arg-p (arg)
+           (not (or (atom arg)
+                    (eq (caar arg) '%log)
+                    (not (isinop arg '%log))
+                    ;; Product of a number with a logarithm e.g. 3*log(x)
+                    (and (eq (caar arg) 'mtimes)
+                         (null (cdddr arg))
+                         (mnump (cadr arg))
+                         (not (atom (caddr arg)))
+                         (eq (caar (caddr arg)) '%log))))))
+    (and (eq (caar e) 'mplus)
+         (not (find-if #'lgc-nonsimple-arg-p (cdr e))))))
 
+;; Sort the argument so that coefficients come before logarithms and logarithms
+;; come before everything else.
 (defun lgcsort (e)
-  (let (genvar varlist ($keepfloat t) vl e1)
-    (newvar e)
-    (setq vl (do ((vl varlist (cdr vl)) (logs) (notlogs) (decints))
-		 ((null vl)
-		  (setq logs (sort logs #'great))
-		  (nreconc decints (nconc logs (nreverse notlogs))))
-	       (cond ((and (not (atom (car vl))) (eq (caaar vl) '%log))
-		      (setq logs (cons (car vl) logs)))
-		     ((logconcoeffp (car vl))
-		      (setq decints (cons (car vl) decints)))
-		     (t (setq notlogs (cons (car vl) notlogs))))))
-    (setq e1 (ratdisrep (ratrep e vl)))
-    (if (alike1 e e1) e e1)))
+  (let ((logs) (notlogs) (decints) (varlist))
+    ;; Split the variables in E into logs, notlogs and coefficients. The list of
+    ;; variables is calculated by NEWVAR (and stored in the special variable
+    ;; VARLIST, which is why we have to bind it above).
+    (dolist (var (newvar e))
+      (cond
+        ((and (not (atom var)) (eq (caar var) '%log)) (push var logs))
+        ((logconcoeffp var) (push var decints))
+        (t (push var notlogs))))
+    (let* ((vl (nreconc decints (nconc (sort logs #'great)
+                                       (nreverse notlogs))))
+           (e1 (ratdisrep (ratrep e vl))))
+      (if (alike1 e e1) e e1))))
 
-(defun lgccheck (e)
+;; lgcreciprocal performs the transformation log(1/x) => -log(x)
+(defun lgcreciprocal (e)
   (let (num denom)
-    (cond ((atom e) e)
-	  ((and (eq (caar e) '%log)
-		(setq num (member ($num (cadr e)) '(1 -1) :test #'equal))
-		(not (equal (setq denom ($denom (cadr e))) 1)))
-	   (list '(mtimes simp) -1
-		 (list '(%log simp) (if (= (car num) 1) denom (neg denom)))))
-	  (t (recur-apply #'lgccheck e)))))
+    (cond
+      ((atom e) e)
+      ((and (eq (caar e) '%log)
+            (setq num (member ($num (cadr e)) '(1 -1) :test #'equal))
+            (not (equal (setq denom ($denom (cadr e))) 1)))
+       (list '(mtimes simp) -1
+             (list '(%log simp) (if (= (car num) 1) denom (neg denom)))))
+      (t (recur-apply #'lgcreciprocal e)))))
 
 (defun logconcoeffp (e)
-  (if $logconcoeffp (let ((*e* e)) (is '(($logconcoeffp) *e*))) (maxima-integerp e)))
+  (if $logconcoeffp
+      (is `(($logconcoeffp) ,e))
+      (maxima-integerp e)))
 
 ;;;; RTCON
 
-(declare-top (special $radexpand $domain radpe))
+(declare-top (special $radexpand $domain))
 
 (defmvar $rootsconmode t)
 
-(defun $rootscontract (e)	       ; E is assumed to be simplified
+(defmfun $rootscontract (e)	       ; E is assumed to be simplified
   (let ((radpe (and $radexpand (not (eq $radexpand '$all)) (eq $domain '$real)))
 	($radexpand nil))
-    (rtcon e)))
+    (rtcon e radpe)))
 
-(defun rtcon (e)
+(defun rtcon (e radpe)
   (cond ((atom e) e)
 	((eq (caar e) 'mtimes)
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-; This code does the substitution %i -> (-1)*(-1)^(-1/2) for expressions
-; like %i/expr. In most cases this causes a wrong result, e.g. 
-; bug ID: 2820202 - rootscontract(%i/2) --> -%i/2. The only case this codes 
-; works is for expressions like %i/sqrt(x) --> -1/sqrt(x). It is doubtful that 
-; this simplification is useful.
-; To get correct results in general, this code is commented out. DK 08/2009.
-; 
-;	 (if (and (not (free e '$%i))
-;		  (let ((num ($num e)))
-;		    (and (not (alike1 e num))
-;			 (or (eq num '$%i)
-;			     (and (not (atom num)) (member '$%i num :test #'eq)
-;				  (member '$%i (rtcon num) :test #'eq))))))
-;	     (setq e (list* (car e) -1 '((mexpt) -1 ((rat simp) -1 2))
-;			    (delete '$%i (copy-list (cdr e)) :count 1 :test #'eq))))
 	 (do ((x (cdr e) (cdr x)) (roots) (notroots) (y))
 	     ((null x)
 	      (cond ((null roots) (subst0 (cons '(mtimes) (nreverse notroots)) e))
 		    (t (if $rootsconmode
-			   (destructuring-let (((min gcd lcm) (rtc-getinfo roots)))
+			   (multiple-value-bind (min gcd lcm)
+                               (rtc-getinfo roots)
 			     (cond ((and (= min gcd) (not (= gcd 1))
-					 (not (= min lcm))
-					 (not (eq $rootsconmode '$all)))
+                                       (not (= min lcm))
+                                       (not (eq $rootsconmode '$all)))
 				    (setq roots
 					  (rt-separ
 					   (list gcd
 						 (rtcon
 						  (rtc-fixitup
 						   (rtc-divide-by-gcd roots gcd)
-						   nil))
+						   nil) radpe)
 						 1)
 					   nil)))
 				   ((eq $rootsconmode '$all)
@@ -379,16 +432,16 @@
 		 ((and (eq (caaar x) 'mexpt) (ratnump (setq y (caddar x))))
 		  (setq roots (rt-separ (list (caddr y)
 					      (list '(mexpt)
-						    (rtcon (cadar x)) (cadr y)))
+						    (rtcon (cadar x) radpe) (cadr y)))
 					roots)))
 
 		 ((and radpe (eq (caaar x) 'mabs))
-		  (setq roots (rt-separ (list 2 `((mexpt) ,(rtcon (cadar x)) 2) 1)
+		  (setq roots (rt-separ (list 2 `((mexpt) ,(rtcon (cadar x) radpe) 2) 1)
 					roots)))
-		 (t (setq notroots (cons (rtcon (car x)) notroots))))))
+		 (t (setq notroots (cons (rtcon (car x) radpe) notroots))))))
 	((and radpe (eq (caar e) 'mabs))
-	 (power (power (rtcon (cadr e)) 2) '((rat simp) 1 2)))
-	(t (recur-apply #'rtcon e))))
+	 (power (power (rtcon (cadr e) radpe) 2) '((rat simp) 1 2)))
+	(t (recur-apply #'(lambda (x) (rtcon x radpe)) e))))
 
 ;; RT-SEPAR separates like roots into their appropriate "buckets",
 ;; where a bucket looks like:
@@ -407,11 +460,14 @@
       (push (list '(mexpt) (muln (cdar x) nil) (quotient lcm (caar x)))
 	    root1))))
 
-(defun rtc-getinfo (llist)
-  (let ((m (caar llist)) (g (caar llist)) (l (caar llist)))
-    (do ((x (cdr llist) (cdr x)))
-	((null x) (list m g l))
-      (setq m (min m (caar x)) g (gcd g (caar x)) l (lcm l (caar x))))))
+(defun rtc-getinfo (list)
+  (let ((m (caar list))
+        (g (caar list))
+        (l (caar list)))
+    (dolist (x (cdr list) (values m g l))
+      (setq m (min m (car x))
+            g (gcd g (car x))
+            l (lcm l (car x))))))
 
 (defun rtc-fixitup (roots notroots)
   (mapcar #'(lambda (x) (rplacd x (list (sratsimp (muln (cdr x) (not $rootsconmode))))))
@@ -444,6 +500,9 @@
 
 (declare-top (special $numer $logarc $trigsign))
 
+;; atan2 distributes over lists, matrices, and equations
+(defprop $atan2 (mlist $matrix mequal) distribute_over)
+
 (defun simpatan2 (expr vestigial z)     ; atan2(y,x) ~ atan(y/x)
   (declare (ignore vestigial))
   (twoargcheck expr)
@@ -456,7 +515,7 @@
            (and (or (numberp x) (ratnump x))       ; both numbers
                 (or (numberp y) (ratnump y))       ; ... but not bigfloats
                 (or $numer (floatp x) (floatp y))) ; at least one float
-           (atan2 ($float y) ($float x)))
+           (atan ($float y) ($float x)))
           ( ;; bfloat contagion
            (and (mnump x)
                 (mnump y)
@@ -501,7 +560,7 @@
                              ((member signx '($pos $pz)) (div '$%pi -4)))))))
           ($logarc
            (logarc '%atan2 (list ($logarc y) ($logarc x))))
-          ((and $trigsign (mminusp* y))
+          ((and $trigsign (eq t (mminusp y)))
            (neg (take '($atan2) (neg y) x)))
           ;; atan2(y,x) = atan(y/x) + pi sign(y) (1-sign(x))/2
           ((eq signx '$pos)
@@ -518,16 +577,14 @@
 
 ;;;; ARITHF
 
-(declare-top (special lnorecurse))
-
-(defmfun $fibtophi (e)
+(defmfun $fibtophi (e &optional (lnorecurse nil))
   (cond ((atom e) e)
 	((eq (caar e) '$fib)
-	 (setq e (cond (lnorecurse (cadr e)) (t ($fibtophi (cadr e)))))
+	 (setq e (cond (lnorecurse (cadr e)) (t ($fibtophi (cadr e) lnorecurse))))
 	 (let ((phi (meval '$%phi)))
 	   (div (add2 (power phi e) (neg (power (add2 1 (neg phi)) e)))
 		(add2 -1 (mul2 2 phi)))))
-	(t (recur-apply #'$fibtophi e))))
+	(t (recur-apply #'(lambda (x) ($fibtophi x lnorecurse)) e))))
 
 (defmspec $numerval (l) (setq l (cdr l))
 	  (do ((l l (cddr l)) (x (ncons '(mlist simp)))) ((null l) x)
@@ -540,21 +597,20 @@
 	    (add2lnc (car l) $props)
 	    (nconc x (ncons (car l)))))
 
-(declare-top (special powers var depvar))
+(let (my-powers)
+  (declare (special my-powers))
 
-(defmfun $derivdegree (e depvar var)
-  (let (powers) (derivdeg1 e) (if (null powers) 0 (maximin powers '$max))))
+  (defmfun $derivdegree (e depvar var)
+    (let (my-powers) (declare (special my-powers)) (derivdeg1 e depvar var) (if (null my-powers) 0 (maximin my-powers '$max))))
 
-(defun derivdeg1 (e)
-  (cond ((or (atom e) (specrepp e)))
-	((eq (caar e) '%derivative)
-	 (cond ((alike1 (cadr e) depvar)
-		(do ((l (cddr e) (cddr l))) ((null l))
-		  (cond ((alike1 (car l) var)
-			 (return (setq powers (cons (cadr l) powers)))))))))
-	(t (mapc 'derivdeg1 (cdr e)))))
-
-(declare-top (unspecial powers var depvar))
+  (defun derivdeg1 (e depvar var)
+    (cond ((or (atom e) (specrepp e)))
+	  ((eq (caar e) '%derivative)
+	   (cond ((alike1 (cadr e) depvar)
+		  (do ((l (cddr e) (cddr l))) ((null l))
+		    (cond ((alike1 (car l) var)
+			   (return (setq my-powers (cons (cadr l) my-powers)))))))))
+	  (t (mapc #'(lambda (x) (derivdeg1 x depvar var)) (cdr e))))))
 
 ;;;; BOX
 
@@ -573,7 +629,7 @@
       (list '(mlabox) e (box-label l))
       (list '(mbox) e)))
 
-(defmfun box (e label)
+(defun box (e label)
   (if (eq label t)
       (list '(mbox) e)
       ($box e (car label))))
@@ -583,22 +639,18 @@
       x
       (coerce (mstring x) 'string)))
 
-(declare-top (special label))
-
 (defmfun $rembox (e &optional (l nil l?))
   (let ((label (if l? (box-label l) '(nil))))
-    (rembox1 e)))
+    (rembox1 e label)))
 
-(defun rembox1 (e)
+(defun rembox1 (e label)
   (cond ((atom e) e)
 	((or (and (eq (caar e) 'mbox)
 		  (or (equal label '(nil)) (member label '($unlabelled $unlabeled) :test #'eq)))
 	     (and (eq (caar e) 'mlabox)
 		  (or (equal label '(nil)) (equal label (caddr e)))))
-	 (rembox1 (cadr e)))
-	(t (recur-apply #'rembox1 e))))
-
-(declare-top (unspecial label))
+	 (rembox1 (cadr e) label))
+	(t (recur-apply #'(lambda (x) (rembox1 x label)) e))))
 
 ;;;; MAPF
 
@@ -608,7 +660,7 @@
   (let ((scanmapp t))
     (resimplify (apply #'scanmap1 (mmapev l)))))
 
-(defmfun scanmap1 (func e &optional (flag nil flag?))
+(defun scanmap1 (func e &optional (flag nil flag?))
   (let ((arg2 (specrepcheck e)) newarg2)
     (cond ((eq func '$rat)
 	   (merror (intl:gettext "scanmap: cannot apply 'rat'.")))
@@ -719,10 +771,17 @@
   (declare (dynamic-extent cols))
   (cond ((not ($matrixp m)) (merror (intl:gettext "addcol: first argument must be a matrix; found ~M") m))
 	((null cols) m)
+	((null (cdr m))
+	 (apply '$addcol (cons (ensure-matrix-column (first cols)) (rest cols))))
 	(t (let ((m ($transpose m)))
 	     (dolist (c cols ($transpose m))
 	       (setq m (addrow m ($transpose c))))))))
 
+(defun ensure-matrix-column (a)
+  (if ($matrixp a) a
+    ;; otherwise must be a MLIST.
+    `(($matrix) ,@(mapcar #'(lambda (e) `((mlist) ,e)) (cdr a)))))
+
 (defun addrow (m r)
   (cond ((not (mxorlistp r)) (merror (intl:gettext "addrow or addcol: argument must be a matrix or list; found ~M") r))
 	((and (cdr m)
@@ -736,10 +795,22 @@
 
 ;;;; ARRAYF
 
+(defun my-nonatomic-expr-p (e)
+  (and (consp e) (consp (car e)) (symbolp (caar e))))
+
+(defun my-lambda-expr-p (e)
+  (and (consp e) (consp (car e)) (eq 'lambda (caar e))))
+
 (defmfun $arraymake (ary subs)
-  (cond ((or (not ($listp subs)) (null (cdr subs)))
+  (cond
+    ;; We go through some gyrations here to allow as wide a range of inputs as possible.
+    ;; Previously $ARRAYMAKE didn't check the first argument at all;
+    ;; this is an attempt at a minimally-restrictive change.
+	((not (or (symbolp ary) ($subvarp ary) (and (my-nonatomic-expr-p ary) (not (my-lambda-expr-p ary)))))
+	 (merror (intl:gettext "arraymake: first argument must be a symbol, subscripted symbol, or nonatomic expression (but not a lambda expression); found: ~M") ary))
+	((or (not ($listp subs)) (null (cdr subs)))
 	 (merror (intl:gettext "arraymake: second argument must be a list of one or more elements; found ~M") subs))
-	((eq (ml-typep ary) 'symbol)
+	((symbolp ary)
 	 (cons (cons (getopr ary) '(array)) (cdr subs)))
 	(t (cons '(mqapply array) (cons ary (cdr subs))))))
 
@@ -751,28 +822,30 @@
   (prog (arra ary)
      (setq arra val)
      (setq ary sym)
-     (if (and arra (or (hash-table-p arra) (arrayp arra)))
+     (if (and arra
+              (or (hash-table-p arra)
+                  (arrayp arra)
+                  (eq (marray-type arra) '$functional)))
 	 (cond ((hash-table-p arra)
 		(let ((dim1 (gethash 'dim1 arra)))
 		  (return (list* '(mlist) '$hash_table (if dim1 1 t)
-				 (loop for u being the hash-keys in arra using (hash-value v)
+				 (loop for u being the hash-keys in arra
 				    unless (eq u 'dim1)
 				    collect
-				    (if (progn v dim1) ;;ignore v
+				    (if dim1
 					u
 					(cons '(mlist simp) u)))))))
 	       ((arrayp arra)
 		(return (let ((dims (array-dimensions arra)))
 			  (list '(mlist) '$declared
 				;; they don't want more info (array-type arra)
-				(length dims)
-				(cons '(mlist) (mapcar #'1- dims)))))))
-	 (let ((gen (mgetl sym '(hashar array))) ary1)
-	   (cond ((null gen)
-		  (merror (intl:gettext "arrayinfo: ~M is not an array.") ary))
-		 ((mfilep (cadr gen))
-		  (i-$unstore (ncons ary))
-		  (setq gen (mgetl ary '(hashar array)))))
+			        (length dims)
+			        (cons '(mlist) (mapcar #'1- dims))))))
+	       ((eq (marray-type arra) '$functional)
+	        (return (arrayinfo-aux sym (mgenarray-content arra)))))
+	 (let ((gen (safe-mgetl sym '(hashar array))) ary1)
+           (when (null gen)
+             (merror (intl:gettext "arrayinfo: ~M is not an array.") ary))
 	   (setq ary1 (cadr gen))
 	   (cond ((eq (car gen) 'hashar)
 		  (setq ary1 (symbol-array ary1))
@@ -794,31 +867,9 @@
 				  (length (cdr ary1))
 				  (cons '(mlist simp) (mapcar #'1- (cdr ary1)))))))))))
 
-
 ;;;; ALIAS
 
-(declare-top (special aliaslist aliascntr greatorder lessorder))
-
-(defmspec $makeatomic (l)
-  (setq l (cdr l))
-  (do ((l l (cdr l)) (bas) (x))
-      ((null l) '$done)
-    (if (or (atom (car l))
-	    (not (or (setq x (member (caaar l) '(mexpt mncexpt) :test #'eq))
-		     (member 'array (cdaar l) :test #'eq))))
-	(improper-arg-err (car l) '$makeatomic))
-    (if x
-	(setq bas (cadar l) x (and (atom (caddar l)) (caddar l)))
-	(setq bas (caaar l) x (and (atom (cadar l)) (cadar l))))
-    (unless (atom bas)
-      (improper-arg-err (car l) '$makeatomic))
-    (setq aliaslist
-	  (cons (cons (car l)
-		      (implode
-		       (nconc (exploden bas)
-			      (or (and x (exploden x)) (ncons '| |))
-			      (cons '$ (mexploden (incf aliascntr))))))
-		aliaslist))))
+(declare-top (special greatorder lessorder))
 
 (defmspec $ordergreat (l)
   (if greatorder (merror (intl:gettext "ordergreat: reordering is not allowed.")))
@@ -847,7 +898,10 @@
 
 ;;;; CONCAT
 
-(defun $concat (&rest l)
+(defmfun $concat (&rest l)
+  "Concatenates its arguments.
+The arguments must evaluate to atoms. The return value is a symbol if
+the first argument is a symbol and a string otherwise."
   (when (null l)
     (merror (intl:gettext "concat: there must be at least one argument.")))
   (let ((result-is-a-string (or (numberp (car l)) (stringp (car l)))))
