@@ -14,20 +14,20 @@
 
 (load-macsyma-macros rzmac)
 
-(declare-top (special var %p%i varlist plogabs half%pi nn* dn* $factlim
+(declare-top (special var %p%i varlist plogabs half%pi nn* dn* $factlim sign
                       $beta_expand))
 
 (defmvar $gammalim 10000
   "Controls simplification of gamma for rational number arguments.")
 
-(defvar $gamma_expand nil
+(defmvar $gamma_expand nil
   "Expand gamma(z+n) for n an integer when T.") 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;; Implementation of the plog function
 
-(defmfun simpplog (x vestigial z)
+(defun simpplog (x vestigial z)
   (declare (ignore vestigial))
   (prog (varlist dd check y)
      (oneargcheck x)
@@ -64,7 +64,7 @@
        ((and (equal ($imagpart (setq z (div* x '$%i))) 0)
 	     (setq y ($asksign z)))
 	(cond
-	  ((equal y '$zero) (merror (intl:gettext "plog(0) is undefined.")))
+	  ((equal y '$zero) (merror (intl:gettext "plog: plog(0) is undefined.")))
 	  (t (cond ((eq y '$pos) (setq y 1))
 		   ((eq y '$neg) (setq y -1)))
 	     (return (add2* (simpln (list '(%log)
@@ -90,10 +90,13 @@
 		    ((equal r '$neg) (return (simplify '$%pi)))
 		    (t (merror (intl:gettext "plog: encountered atan(0/0).")))))
 	     ((and (among '%cos r) (among '%sin i))
-	      (setq var 'xz)
-	      (numden (div* r i))
-	      (cond ((and (eq (caar nn*) '%cos) (eq (caar dn*) '%sin))
-		     (return (cadr nn*))))))
+              ;; genvar and varlist, used by the rational function system,
+              ;; are bound in order to prevent the symbol 'xz from leaking
+              ;; out of this function.
+              (let ((var 'xz) genvar varlist)
+		(numden (div* r i))
+		(cond ((and (eq (caar nn*) '%cos) (eq (caar dn*) '%sin))
+		       (return (cadr nn*)))))))
        (setq a ($sign r) b ($sign i))
        (cond ((eq a '$pos) (setq a 1))
 	     ((eq a '$neg) (setq a -1))
@@ -120,7 +123,7 @@
 ;;; Implementation of the Binomial coefficient
 
 ;; Verb function for the Binomial coefficient
-(defun $binomial (x y)
+(defmfun $binomial (x y)
   (simplify (list '(%binomial) x y)))
 
 ;; Binomial has Mirror symmetry
@@ -172,7 +175,7 @@
       (ncons u)
       (list* u (list '(mexpt) v -1) (bincomp1 (add2 -1 u) (1- v)))))
 
-(defmfun binocomp (u v) 
+(defun binocomp (u v) 
   (prog (ans) 
      (setq ans 1)
      loop (if (zerop v) (return ans))
@@ -180,6 +183,24 @@
      (setq u (addk -1 u) v (1- v))
      (go loop)))
 
+;;; gradient of binomial
+
+(defprop %binomial 
+		 ((a b)
+			((mtimes) -1 ((%binomial) a b)
+              ((mplus)
+                 ((mtimes) -1
+                 ((mqapply) (($psi array) 0) ((mplus) 1 a)))
+                 ((mqapply) (($psi array) 0)
+                 ((mplus) 1 a ((mtimes) -1 b)))))
+		  
+		    ((mtimes) -1 ((%binomial) a b)
+               ((mplus)
+                 ((mtimes) -1
+                 ((mqapply) (($psi array) 0)
+                 ((mplus) 1 a ((mtimes) -1 b))))
+                 ((mqapply) (($psi array) 0) ((mplus) 1 b))))) grad)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;; Implementation of the Beta function
@@ -191,7 +212,7 @@
 ;;; The Beta function has mirror symmetry
 (defprop $beta t commutes-with-conjugate)
 
-(defmfun simpbeta (x vestigial z &aux check)
+(defun simpbeta (x vestigial z &aux check)
   (declare (ignore vestigial))
   (twoargcheck x)
   (setq check x)
@@ -359,7 +380,7 @@
 
 ;;; Implementation of the Gamma function
 
-(defmfun simpgamma (x vestigial z)
+(defun simpgamma (x vestigial z)
   (declare (ignore vestigial))
   (oneargcheck x)
   (let ((j (simpcheck (cadr x) z)))
@@ -378,21 +399,40 @@
            ;; Adding 4 digits in the call to bffac. For $fpprec up to about 256
            ;; and an argument up to about 500.0 the accuracy of the result is
            ;; better than 10^(-$fpprec).
-	   (let ((result (mfuncall '$bffac (m+ ($bfloat j) -1) (+ $fpprec 4))))
-	     ;; bigfloatp will round the result to the correct fpprec
-	     (bigfloatp result)))
+	   (let ((z (bigfloat:to ($bfloat j))))
+	     (cond
+	       ((bigfloat:<= (bigfloat:abs z) (bigfloat:sqrt (bigfloat:epsilon z)))
+		;; For small z, use gamma(z) = gamma(z+1)/z = z!/z
+		(div (mfuncall '$bffac
+			       ($bfloat j)
+			       (+ $fpprec 4))
+		     ($bfloat j)))
+	       (t
+		(let ((result (mfuncall '$bffac (m+ ($bfloat j) -1) (+ $fpprec 4))))
+		  ;; bigfloatp will round the result to the correct fpprec
+		  (bigfloatp result))))))
 	  ((complex-float-numerical-eval-p j)
            (complexify (gamma-lanczos (complex ($float ($realpart j))
                                                ($float ($imagpart j))))))
           ((complex-bigfloat-numerical-eval-p j)
-           ;; Adding 4 digits in the call to cbffac. See comment above.
-	   (let ((result
-		  (mfuncall '$cbffac 
-			    (add -1 ($bfloat ($realpart j)) 
-				 (mul '$%i ($bfloat ($imagpart j))))
-			    (+ $fpprec 4))))
-	     (add (bigfloatp ($realpart result))
-		  (mul '$%i (bigfloatp ($imagpart result))))))
+	   (let ((z (bigfloat:to ($bfloat j))))
+	     (cond
+	       ((bigfloat:<= (bigfloat:abs z)
+			     (bigfloat:sqrt (bigfloat:epsilon z)))
+		;; For small z, use gamma(z) = gamma(z+1)/z = z!/z
+		(to (bigfloat:/ (bigfloat:to (mfuncall '$cbffac 
+						       (to z)
+						       (+ $fpprec 4)))
+				z)))
+	       (t
+		;; Adding 4 digits in the call to cbffac. See comment above.
+		(let ((result
+		       (mfuncall '$cbffac 
+				 (add -1 ($bfloat ($realpart j)) 
+				      (mul '$%i ($bfloat ($imagpart j))))
+				 (+ $fpprec 4))))
+		  (add (bigfloatp ($realpart result))
+		       (mul '$%i (bigfloatp ($imagpart result)))))))))
           ((taylorize (mop x) (cadr x)))
           ((eq j '$inf) '$inf) ; Simplify to $inf to be more consistent.
           ((and $gamma_expand
@@ -431,6 +471,17 @@
            (gammared j))
 	  (t (eqtest (list '(%gamma) j) x)))))
 
+;; A sign function for gamma(x); when x > 0 return pos; when x < 0 or x > 0, return pn;
+;;; otherwise, return pnz (that is, nothing known).
+(defun gamma-sign (x)
+   (let ((sgn ($csign (second x)))) ;; careful! x = ((%gamma) XXX)
+		(setq sign
+			  (cond ((eql sgn '$pos) '$pos)
+				    ((or (eql sgn '$neg) (eql sgn '$pn)) '$pn)
+				    (t '$pnz)))))
+
+(putprop '%gamma #'gamma-sign 'sign-function)
+
 (defun gamma (y) ;;; numerical evaluation for 0 < y < 1
   (prog (sum coefs)
      (setq coefs (list 0.035868343 -0.193527817 0.48219939
@@ -510,51 +561,63 @@
 			 -.26190838401581408670e-4
 			 .36899182659531622704e-5))))
     (declare (type (simple-array flonum (15)) c))
-    (if (minusp (realpart z))
-	;; Use the reflection formula
-	;; -z*Gamma(z)*Gamma(-z) = pi/sin(pi*z)
-	;; or
-	;; Gamma(z) = pi/z/sin(pi*z)/Gamma(-z)
-	;;
-	;; If z is a negative integer, Gamma(z) is infinity.  Should
-	;; we test for this?  Throw an error?
-        ;; The test must be done by the calling routine.
-	(/ (float pi)
-	   (* (- z) (sin (* (float pi) z))
-	      (gamma-lanczos (- z))))
-	(let* ((z (- z 1))
-	       (zh (+ z 1/2))
-	       (zgh (+ zh 607/128))
-	       (ss 
-		(do ((sum 0.0)
-		     (pp (1- (length c)) (1- pp)))
-		    ((< pp 1)
-		     sum)
-		  (incf sum (/ (aref c pp) (+ z pp))))))
-          (let ((result 
-                 ;; We check for an overflow. The last positive value in 
-                 ;; double-float precicsion for which Maxima can calculate 
-                 ;; gamma is ~171.6243 (CLISP 2.46 and GCL 2.6.8)
-                 (ignore-errors
-		   (let ((zp (expt zgh (/ zh 2))))
-		     (* (sqrt (float (* 2 pi)))
-			(+ ss (aref c 0))
-			(* (/ zp (exp zgh)) zp))))))
-            (cond ((null result)
-                   ;; No result. Overflow.
-                   (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
-                  ((or (float-nan-p (realpart result))
-                       (float-inf-p (realpart result)))
-                   ;; Result, but beyond extreme values. Overflow.
-                   (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
-                  (t result)))))))
+    (cond
+      ((minusp (realpart z))
+       ;; Use the reflection formula
+       ;; -z*Gamma(z)*Gamma(-z) = pi/sin(pi*z)
+       ;; or
+       ;; Gamma(z) = pi/z/sin(pi*z)/Gamma(-z)
+       ;;
+       ;; If z is a negative integer, Gamma(z) is infinity.  Should
+       ;; we test for this?  Throw an error?
+       ;; The test must be done by the calling routine.
+       (/ (float pi)
+	  (* (- z) (sin (* (float pi) z))
+	     (gamma-lanczos (- z)))))
+      ((<= (abs z) (sqrt flonum-epsilon))
+       ;; For |z| small, use Gamma(z) = Gamma(z+1)/z
+       (/ (gamma-lanczos (+ 1 z))
+	  z))
+      (t
+       (let* ((z (- z 1))
+	      (zh (+ z 1/2))
+	      (zgh (+ zh 607/128))
+	      (ss 
+	       (do ((sum 0.0)
+		    (pp (1- (length c)) (1- pp)))
+		   ((< pp 1)
+		    sum)
+		 (incf sum (/ (aref c pp) (+ z pp))))))
+	 (let ((result 
+		;; We check for an overflow. The last positive value in 
+		;; double-float precicsion for which Maxima can calculate 
+		;; gamma is ~171.6243 (CLISP 2.46 and GCL 2.6.8)
+		(ignore-errors
+		  (let ((zp (expt zgh (/ zh 2))))
+		    (* (sqrt (float (* 2 pi)))
+		       (+ ss (aref c 0))
+		       (* (/ zp (exp zgh)) zp))))))
+	   (cond ((null result)
+		  ;; No result. Overflow.
+		  (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
+		 ((or (float-nan-p (realpart result))
+		      (float-inf-p (realpart result)))
+		  ;; Result, but beyond extreme values. Overflow.
+		  (merror (intl:gettext "gamma: overflow in GAMMA-LANCZOS.")))
+		 (t result))))))))
 
 (defun gammafloat (a)
   (let ((a (float a)))
     (cond ((minusp a)
+	   ;; Reflection formula to make it positive: gamma(x) =
+	   ;; %pi/sin(%pi*x)/x/gamma(-x)
 	   (/ (float (- pi))
 	      (* a (sin (* (float pi) a)))
 	      (gammafloat (- a))))
+	  ((<= a (sqrt flonum-epsilon))
+	   ;; Use gamma(x) = gamma(1+x)/x when x is very small
+	   (/ (gammafloat (+ 1 a))
+	      a))
 	  ((< a 10)
 	   (slatec::dgamma a))
 	  (t
@@ -677,8 +740,7 @@
 	((mtimesp e) (muln (mapcar '$xthru (cdr e)) nil))
 	((mplusp e) (simplify (comdenom (mapcar '$xthru (cdr e)) t)))
 	((mexptp e) (power ($xthru (cadr e)) (caddr e)))
-	((member (caar e) '(mequal mlist $matrix) :test #'eq)
-	 (cons (car e) (mapcar '$xthru (cdr e))))
+	((mbagp e) (cons (car e) (mapcar '$xthru (cdr e))))
 	(t e))) 
 
 (defun comdenom (l ind) 
@@ -781,7 +843,7 @@
 			   (let (($nolabels nil))
 			     (makelabel $linechar))
 			   *linelabel*)
-			 (set *linelabel* zzz)))
+			 (setf (symbol-value *linelabel*) zzz)))
 	 (nconc sol (ncons *linelabel*))
 	 (cond ($dispflag
 		(setq tim (get-internal-run-time))
