@@ -400,16 +400,16 @@
       (setq +labs nil))
     isp))
 
+;; Return NIL for all non-symbols.
 (defun kindp (x y)
-  (unless (symbolp x)
-    (merror (intl:gettext "declare: argument must be a symbol; found ~M") x))
-  (clear)
-  (beg x 1)
-  (do ((p (dq+) (dq+)))
-      ((null p))
-    (if (eq y p)
-	(return t)
-	(mark+ p (+labs p)))))
+  (when (symbolp x)
+    (clear)
+    (beg x 1)
+    (do ((p (dq+) (dq+)))
+        ((null p))
+        (if (eq y p)
+          (return t)
+          (mark+ p (+labs p))))))
 
 (defun true* (pat)
   (let ((dum (semant pat)))
