@@ -18,6 +18,8 @@
 
 (declare-top (special $linel))
 
+(defmvar $ttyoff nil)
+
 (defmvar $stardisp nil
   "Causes factors of products to be separated by * when displayed.")
 
@@ -56,24 +58,28 @@
 ;; for each window.  Set them here, anyway, so that RETRIEVE can be called from
 ;; top level.  The size of TOP-WINDOW is wired in here.
 
-(setq linel 79. $linel 79. ttyheight 24.)
+(defmvar $linel 79.)
+(defvar linel 79.)
+(defvar ttyheight 24.)
 
 (defvar linearray (make-array 80. :initial-element nil))
 
-(defmfun maxima-display (form &key (stream *standard-output*) )
+(defun maxima-display (form &key (stream *standard-output*) )
   (let ((*standard-output* stream))
     (displa form)))
 
 (defvar *alt-display2d* nil)
 (defvar *alt-display1d* nil)
 
-(defmfun displa (form &aux #+kcl(form form))
-  (if (not #.ttyoff)
+(defun displa (form &aux #+kcl(form form))
+  (if (not $ttyoff)
       (cond ($display2d
 	     (cond (*alt-display2d* (apply *alt-display2d* form ()))
 		   (t
 		    (let ((displayp t)
-			  (linearray (if displayp (make-array 80.) linearray))
+			  (linearray (if displayp
+					 (make-array 80. :initial-element nil)
+					 linearray))
 			  (mratp (checkrat form))
 			  (maxht     1) (maxdp   0) (width   0)
 			  (height    0) (depth   0) (level   0) (size   2)
@@ -109,7 +115,7 @@
 (defun linear-displa (form)
   (fresh-line *standard-output*)
   (mgrind form *standard-output*)
-  (terpri))
+  (mterpri))
 
 (defmvar $display_format_internal nil
   "Setting this TRUE can help give the user a greater understanding
@@ -143,7 +149,7 @@
 
 ;; Referenced externally by RAT;FLOAT.
 
-(defmfun dimension-atom (form result)
+(defun dimension-atom (form result)
   (cond ((and (symbolp form) (get form atom-context))
 	 (funcall (get form atom-context) form result))
 	((stringp form) (dimension-string (makestring form) result))
@@ -154,7 +160,7 @@
 ;; Referenced externally by anyone who wants to display something as
 ;; a funny looking atom, e.g. Trace, Mformat.
 
-(defmfun dimension-string (dummy result &aux crp)
+(defun dimension-string (dummy result &aux crp)
   (setq width 0 height 1 depth 0)
   (do ((l dummy (cdr l)))
       ((null l))
@@ -176,7 +182,7 @@
 			(incf width))
 		      (setq result (rplacd dummy result))))))))
 
-(defmfun makestring (atom)
+(defun makestring (atom)
   (let (dummy)
     (cond ((numberp atom) (exploden atom))
       ((stringp atom)
@@ -204,35 +210,109 @@
   result)
 
 (defun dimension-array (x result)
-  (prog (dummy bas (w 0) (h 0) (d 0) sub)
+  (prog (dummy bas (w-base 0) (h-base 0) (d-base 0))
      (if (eq (caar x) 'mqapply)
-	 (setq dummy (cadr x) x (cdr x))
-	 (setq dummy (caar x)))
+     (setq dummy (cadr x) x (cdr x))
+     (setq dummy (caar x)))
      (cond ((or (not $noundisp) (not (symbolp (caar x)))))
-	   ((and (get (caar x) 'verb) (get (caar x) 'alias))
-	    (push-string "''" result) (setq w 2))
-	   ((and (get (caar x) 'noun) (not (member (caar x) (cdr $aliases) :test #'eq))
-		 (not (get (caar x) 'reversealias)))
-	    (setq result (cons #\' result) w 1)))
-     (setq sub (let ((lop 'mparen)
-		     (rop 'mparen)
-		     (break nil) (size 1))
-		 (dimension-list x nil))
-	   w (+ w width)
-	   h height
-	   d depth)
-     (setq bas (if (and (not (atom dummy)) (member 'array (car dummy) :test #'eq))
-		   (let ((break nil) (right 0)) (dimension-paren dummy result))
-		   (let ((atom-context 'dimension-array))
-		     (dimension dummy result lop 'mfunction nil 0))))
-     (cond ((not (checkfit (setq width (+ w width))))
-	    (return (dimension-function (cons '(subscript) (cons dummy (cdr x))) result)))
-	   ((and (atom (car bas)) (char= #\ (car bas))) ; Adding the test (atom (car bas))
-	    (setq result (cons (cons 0 (cons (- h) sub)) bas) depth (max (+ h d) depth)))
-	   (t (setq result (cons (cons 0 (cons (- (+ depth h)) sub)) bas)
-		    depth (+ h d depth))))
-     (update-heights height depth)
-     (return result)))
+       ((and (get (caar x) 'verb) (get (caar x) 'alias))
+        (push-string "''" result) (setq w-base 2))
+       ((and (get (caar x) 'noun) (not (member (caar x) (cdr $aliases) :test #'eq))
+         (not (get (caar x) 'reversealias)))
+        (setq result (cons #\' result) w-base 1)))
+     (setq
+       bas (if (and (not (atom dummy)) (member 'array (car dummy) :test #'eq))
+             (let ((break nil) (right 0)) (dimension-paren dummy nil))
+             (let ((atom-context 'dimension-array))
+               (dimension dummy nil lop 'mfunction nil 0)))
+       w-base (+ w-base width)
+       h-base height
+       d-base depth)
+     (multiple-value-bind (w00 w01 w10 w11 h00 h01 h10 h11 d00 d01 d10 d11 pre-subscripts-output pre-superscripts-output post-subscripts-output post-superscripts-output)
+       (dimension-indices dummy (cdr x))
+       (declare (special pre-subscripts pre-superscripts post-subscripts post-superscripts))
+       (cond
+         ((not (checkfit (setq width (+ w-base (max w00 w01) (max w10 w11))))) ;; ?? !!
+          (return (dimension-function (cons '(subscript) (cons dummy (cdr x))) result)))
+         ((and (atom (car bas)) (char= #\ (car bas))) ; Adding the test (atom (car bas))
+          (setq result (cons (cons 0 (cons (- h-base) post-subscripts)) bas) depth (max (+ h-base d-base) depth))) ;; ?? !!
+         (t
+           (setq result
+                 (append
+                   (if (and (> w11 0) (> w10 w11)) (list (list (- w10 w11) 0)))
+                   (remove-if #'(lambda (l) (= (length l) 2))
+                              (list
+                                (append (list (- w10) (+ h-base d11)) post-superscripts-output)
+                                (append (list 0 (- h10)) post-subscripts-output)
+                                (append (list 0 0) bas)
+                                (append (list (if (> w00 0) (- w01) 0) (+ h-base d01)) pre-superscripts-output)
+                                (append (list (if (> w01 w00) (- w01 w00) 0) (- h00)) pre-subscripts-output)))
+                   result)
+                 height (+ h-base (max (+ d01 h01) (+ d11 h11)))
+                 depth (+ d-base (max (+ d00 h00) (+ d10 h10))))))
+       (update-heights height depth)
+       (return result))))
+
+(defmvar $known_index_properties '((mlist) $presubscript $presuperscript $postsubscript $postsuperscript))
+
+(defun dimension-indices (base-symbol indices)
+  (let
+    ((display-indices (safe-mget base-symbol 'display-indices))
+     (separator (let ((x (safe-$get base-symbol '$display_index_separator)))
+                  (if (or (null x) (stringp x)) x (coerce (mstring x) 'string)))))
+    (if (and (> (length display-indices) 0) (not (= (length display-indices) (length indices))))
+      ;; Ignore DISPLAY-INDICES if it's nonempty and not the same size as INDICES.
+      (setq display-indices nil))
+    (let (pre-subscripts pre-superscripts post-subscripts post-superscripts)
+      (declare (special pre-subscripts pre-superscripts post-subscripts post-superscripts))
+      (if display-indices
+        (setq pre-subscripts (extract-indices indices display-indices '$presubscript)
+              pre-superscripts (extract-indices indices display-indices '$presuperscript)
+              post-subscripts (extract-indices indices display-indices '$postsubscript)
+              post-superscripts (extract-indices indices display-indices '$postsuperscript))
+        (setq pre-subscripts nil
+              pre-superscripts nil
+              post-subscripts indices
+              post-superscripts nil))
+      (let (pre-subscripts-output pre-superscripts-output post-subscripts-output post-superscripts-output
+            (w00 0) (w01 0) (w10 0) (w11 0) (h00 0) (h01 0) (h10 0) (h11 0) (d00 0) (d01 0) (d10 0) (d11 0))
+        (if pre-subscripts
+          (setq pre-subscripts-output
+                (let ((lop 'mparen) (rop 'mparen) (break nil) (size 1))
+                  (dimension-list (cons '(mlist) pre-subscripts) nil separator))
+                w00 width
+                h00 height
+                d00 depth))
+        (if pre-superscripts
+          (setq pre-superscripts-output
+                (let ((lop 'mparen) (rop 'mparen) (break nil) (size 1))
+                  (dimension-list (cons '(mlist) pre-superscripts) nil separator))
+                w01 width
+                h01 height
+                d01 depth))
+        (if post-subscripts
+          (setq post-subscripts-output
+                (let ((lop 'mparen) (rop 'mparen) (break nil) (size 1))
+                  (dimension-list (cons '(mlist) post-subscripts) nil separator))
+                w10 width
+                h10 height
+                d10 depth))
+        (if post-superscripts
+          (setq post-superscripts-output
+                (let ((lop 'mparen) (rop 'mparen) (break nil) (size 1))
+                  (dimension-list (cons '(mlist) post-superscripts) nil separator))
+                w11 width
+                h11 height
+                d11 depth))
+        (values w00 w01 w10 w11 h00 h01 h10 h11 d00 d01 d10 d11 pre-subscripts-output pre-superscripts-output post-subscripts-output post-superscripts-output)))))
+
+(defun extract-indices (indices display-indices index-flag)
+  (remove nil
+          (loop for i from 0 to (length display-indices)
+                collect (if (eq (nth i display-indices) index-flag) (nth i indices)))))
+
+(defun safe-$get (x y)
+  (and (symbolp x) ($get x y)))
 
 (defun dimension-function (x result)
   (prog (fun (w 0) (h 0) (d 0))
@@ -256,7 +336,7 @@
 		    width (+ 2 w width) height (max h height) depth (max d depth))))
      (return result)))
 
-(defmfun dimension-prefix (form result)
+(defun dimension-prefix (form result)
   (prog (dissym (symlength 0))
      (setq dissym (safe-get (caar form) 'dissym)
 	   symlength (length dissym))
@@ -264,21 +344,21 @@
 	   width (+ symlength width))
      (return result)))
 
-(defun dimension-list (form result)
+(defun dimension-list (form result &optional separator)
   (prog ((w 0) (h 0) (d 0))
      (setq result (dimension (cadr form) result lop 'mcomma 0 right)
 	   w width h height d depth)
      (do ((l (cddr form) (cdr l)))
 	 ((null l))
-       (push-string ", " result)
-       (incf w 2)
+       (push-string (or separator ", ") result)
+       (incf w (if separator (length separator) 2))
        (checkbreak result w)
        (setq result (dimension (car l) result 'mcomma 'mcomma w right)
 	     w (+ w width) h (max h height) d (max d depth)))
      (setq width w height h depth d)
      (return result)))
 
-(defmfun dimension-infix (form result)
+(defun dimension-infix (form result)
   (unless (= (length (cdr form)) 2)
     (return-from dimension-infix (dimension-function form result)))
   (prog (dissym (symlength 0) (w 0) (h 0) (d 0))
@@ -296,7 +376,7 @@
 	   depth (max d depth))
      (return result)))
 
-(defmfun dimension-nary (form result)
+(defun dimension-nary (form result)
   ;; If only 0 or 1 arguments, then print "*"() or "*"(A)
   (cond ((null (cddr form))
 	 (dimension-function form result))
@@ -410,7 +490,7 @@
       (dimension-paren form result)
       (dimension form result lop rop w right)))
 
-(defmfun dimension-postfix (form result)
+(defun dimension-postfix (form result)
   (prog (dissym (symlength 0))
      (setq dissym (safe-get (caar form) 'dissym)
 	   symlength (length dissym))
@@ -418,7 +498,7 @@
 	   width (+ symlength width))
      (return (revappend dissym result))))
 
-(defmfun dimension-nofix (form result)
+(defun dimension-nofix (form result)
   (setq form (safe-get (caar form) 'dissym)
 	width (length form))
   (revappend form result))
@@ -440,7 +520,7 @@
 	      (setq width (+ (length (cdr dissym)) symlength width))
 	      (return (revappend (cdr dissym) result))))))
 
-(defmfun dimension-superscript (form result)
+(defun dimension-superscript (form result)
   (prog (exp (w 0) (h 0) (d 0) bas)
      (setq exp (let ((size 1))
 		 (dimension (caddr form) nil 'mparen 'mparen nil 0))
@@ -448,7 +528,9 @@
 	   h height
 	   d depth)
      (cond ((and (not (atom (cadr form))) (member 'array (cdaadr form) :test #'eq))
-	    (prog (sub (w2 0) (h2 0) (d2 0))
+            (if (safe-mget (caaadr form) 'display-indices)
+              (return (dimension-superscript `(,(first form) ((mprogn) ,(second form)) ,(third form)) result))
+	    (progn (prog (sub (w2 0) (h2 0) (d2 0))
 	       (setq sub (if (eq 'mqapply (caaadr form))
 			     (cdadr form) (cadr form)))
 	       (setq sub (let ((lop 'mparen) (break nil) (size 1))
@@ -463,7 +545,7 @@
 	       (setq result (cons (list (- (max w w2) w2) 0) result)
 		     width (+ width (max w w2)) height (+ h d height) depth (+ d2 h2 depth)))
 	    (update-heights height depth)
-	    (return result))
+	    (return result))))
 	   ((and (atom (caddr form))
 		 (not (atom (cadr form)))
 		 (not (safe-get (caaadr form) 'dimension))
@@ -482,20 +564,20 @@
 	      (return result)))))
 
 (defun dsumprod (form result d-form sw sh sd)
-  (prog (dummy (w 0) (h 0) (d 0) dummy2 (lsum (eq (caar form) '%lsum)))
+  (prog (str to dummy (w 0) (h 0) (d 0) dummy2 (lsum (eq (caar form) '%lsum)))
      (setq dummy2 (dimension (caddr form) nil 'mparen 'mequal nil 0)
 	   w width
 	   h height
 	   d depth)
      (if lsum
-	 (push-string " in "  dummy2)
-	 (push-string " = " dummy2))
+	 (setq str " in " to "")
+	 (setq str " = "  to (cadr (cdddr form))))
+     (push-string str dummy2)
      (setq dummy2 (dimension (cadddr form) dummy2 'mequal 'mparen nil 0)
-	   w (+ 3 w width)
+	   w (+ (length str) w width)
 	   h (max h height)
 	   d (max d depth))
-     (or lsum
-	 (setq dummy (dimension (cadr (cdddr form)) nil 'mparen 'mparen nil 0)))
+     (setq dummy (dimension to nil 'mparen 'mparen nil 0))
      (unless (checkfit (max w width))
        (return (dimension-function form result)))
      (setq dummy2 (cons (cons (- sw) (cons (- (+ sd h)) dummy2)) (cons d-form result)))
@@ -517,11 +599,17 @@
      (return dummy)))
 
 (displa-def bigfloat  dim-bigfloat)
+(displa-def %bigfloat  dim-bigfloat)
 (displa-def mquote    dimension-prefix "'")
+(displa-def %mquote    dimension-prefix "'")
 (displa-def msetq     dimension-infix  " : ")
+(displa-def %msetq    dimension-infix  " : ")
 (displa-def mset      dimension-infix  " :: ")
+(displa-def %mset     dimension-infix  " :: ")
 (displa-def mdefine   dim-mdefine      " := ")
+(displa-def %mdefine  dim-mdefine      " := ")
 (displa-def mdefmacro dim-mdefine      " ::= ")
+(displa-def %mdefmacro dim-mdefine      " ::= ")
 
 (defun dim-mdefine (form result)
   (let (($noundisp t)
@@ -532,14 +620,18 @@
 		     result)))
 
 (displa-def mfactorial dimension-postfix "!")
+(displa-def %mfactorial dimension-postfix "!")
 (displa-def mexpt      dimension-superscript)
+(displa-def %mexpt     dimension-superscript)
 (displa-def mncexpt    dim-mncexpt "^^")
+(displa-def %mncexpt   dim-mncexpt "^^")
 
 (defun dim-mncexpt (form result)
   (dimension-superscript (list '(mncexpt) (cadr form) (cons '(mangle) (cddr form)))
 			 result))
 
 (displa-def mnctimes dimension-nary " . ")
+(displa-def %mnctimes dimension-nary " . ")
 
 (displa-def %product dim-%product 115.)
 
@@ -547,6 +639,7 @@
   (dsumprod form result '(d-prodsign) 5 3 1))
 
 (displa-def rat dim-rat "/")
+(displa-def %rat dim-rat "/")
 
 (defun dim-rat (form result)
   (if $pfeformat
@@ -554,6 +647,7 @@
       (dim-mquotient form result)))
 
 (displa-def mquotient dim-mquotient "/")
+(displa-def %mquotient dim-mquotient "/")
 
 (defun dim-mquotient (form result)
   (unless (= (length (cdr form)) 2)
@@ -596,6 +690,7 @@
   result)
 
 (displa-def mtimes dimension-nary " ")
+(displa-def %mtimes dimension-nary " ")
 
 ;; This code gets run when STARDISP is assigned a value.
 
@@ -717,6 +812,7 @@
      (return result)))
 
 (displa-def mplus  dim-mplus)
+(displa-def %mplus  dim-mplus)
 (defprop munaryplus (#\+ #\space) dissym)
 
 (defun dim-mplus (form result)
@@ -754,6 +850,7 @@
 		      (checkbreak result w)))))))
 
 (displa-def mminus dim-mminus)
+(displa-def %mminus dim-mminus)
 (defprop munaryminus (#\- #\space) dissym)
 (def-rbp munaryminus 134)
 (def-rbp munaryminus 100)
@@ -839,16 +936,26 @@
 ;; sets on consoles which have them.
 
 (displa-def marrow    dimension-infix  " -> " 80. 80.)
+(displa-def %marrow   dimension-infix  " -> " 80. 80.)
 (displa-def mgreaterp dimension-infix  " > ")
+(displa-def %mgreaterp dimension-infix  " > ")
 (displa-def mgeqp     dimension-infix  " >= ")
+(displa-def %mgeqp    dimension-infix  " >= ")
 (displa-def mequal    dimension-infix  " = ")
+(displa-def %mequal   dimension-infix  " = ")
 (displa-def mnotequal dimension-infix  " # ")
+(displa-def %mnotequal dimension-infix  " # ")
 (displa-def mleqp     dimension-infix  " <= ")
+(displa-def %mleqp    dimension-infix  " <= ")
 (displa-def mlessp    dimension-infix  " < ")
+(displa-def %mlessp   dimension-infix  " < ")
 (displa-def mnot      dimension-prefix "not ")
+(displa-def %mnot     dimension-prefix "not ")
 (displa-def mand      dimension-nary   " and ")
+(displa-def %mand      dimension-nary   " and ")
 (defprop mand dimnary-boolean dimension-nary-helper)
 (displa-def mor	      dimension-nary   " or ")
+(displa-def %mor      dimension-nary   " or ")
 (defprop mor dimnary-boolean dimension-nary-helper)
 (displa-def mcond     dim-mcond)
 (displa-def %mcond    dim-mcond)
@@ -991,10 +1098,15 @@
      (return result)))
 
 (displa-def mprogn dimension-match "(" ")")
+(displa-def %mprogn dimension-match "(" ")")
 (displa-def mlist  dimension-match "[" "]")
+(displa-def %mlist  dimension-match "[" "]")
 (displa-def mangle dimension-match "<" ">")
+(displa-def %mangle dimension-match "<" ">")
 (displa-def mcomma dimension-nary  ", " 10. 10.)
+(displa-def %mcomma dimension-nary  ", " 10. 10.)
 (displa-def mabs   dim-mabs)
+(displa-def %mabs  dim-mabs)
 
 (defun dim-mabs (form result &aux arg bar)
   (setq arg (dimension (cadr form) nil 'mparen 'mparen nil 0))
@@ -1006,6 +1118,7 @@
 	   (cons bar (nconc arg (cons bar result))))))
 
 (displa-def $matrix dim-$matrix)
+(displa-def %matrix dim-$matrix)
 
 (defun dim-$matrix (form result)
   (prog (dmstr rstr cstr consp cols)
@@ -1088,6 +1201,7 @@
   result)
 
 (displa-def mbox dim-mbox)
+(displa-def %mbox dim-mbox)
 
 (defun dim-mbox (form result &aux dummy)
   (setq dummy (dimension (cadr form) nil 'mparen 'mparen nil 0))
@@ -1099,6 +1213,7 @@
 	   result)))
 
 (displa-def mlabox dim-mlabox)
+(displa-def %mlabox dim-mlabox)
 
 (defun dim-mlabox (form result)
   (prog (dummy ch)
@@ -1126,6 +1241,7 @@
      (return result)))
 
 (displa-def mtext dim-mtext 1 1)
+(displa-def %mtext dim-mtext 1 1)
 (defprop mtext dimnary-mtext dimension-nary-helper)
 
 (defun dim-mtext (form result)
@@ -1133,6 +1249,7 @@
       (dimension-nary form result)))
 
 (displa-def mlabel dim-mlabel 0 0)
+(displa-def %mlabel dim-mlabel 0 0)
 (setf (get 'mlabel 'wxxml) 'wxxml-mlable) ;; backwards-compatibility for wxMaxima
 
 (defvar *display-labels-p* t)
@@ -1255,9 +1372,8 @@
 #-(or gcl clisp) (cond ((not (interactive-stream-p *standard-input*)) (fresh-line)))
   (cond
     ;; If console output is disabled, don't output anything.
-    (#.ttyoff)
-    ;; Constant 80. in this test appears to be the size of LINEARRAY.
-    ((> (+ bkptht bkptdp) 80.)
+    ($ttyoff)
+    ((> (+ bkptht bkptdp) (length linearray))
      ;; I suppose we could reallocate LINEARRAY to some larger size and keep going here ...
      (merror (intl:gettext "display: expression is too tall to be displayed.")))
     (t
@@ -1285,7 +1401,7 @@
 	   n (car line))
      (setf (aref linearray i) nil)
      (tyotbsp n)
-     (loop for v in (cdr line) do (write-char v))
+     (loop for v in (cdr line) do (write-char v #+(or sbcl cmu) *standard-output*))
      (mterpri)))
 
 ;; Move the cursor over N spaces to the left by outputting spaces.
@@ -1296,7 +1412,7 @@
 (defun tyotbsp (n)
   (do ()
       ((< n 1))
-    (write-char #\space)
+    (write-char #\space #+(or sbcl cmu) *standard-output*)
     (decf n)))
 
 (defun draw-linear (dmstr oldrow oldcol)
