@@ -252,14 +252,14 @@
                       ((stringp x)
                        (tex-string (quote-% (if $stringdisp (concatenate 'string "``" x "''") x))))
                       ((characterp x) (tex-char x))
-		      ((not ($mapatom x))
+		      ((symbolp x) (tex-stripdollar (or (get x 'reversealias) x)))
+		      (t
 		       (let ((x (if (member (marray-type x) '(array hash-table $functional))
 				    ($sconcat x)
 				  (format nil "~A" x))))
-			 (tex-string (quote-chars (if $stringdisp (concatenate 'string "``" x "''") x)
-						  "#$%&_"))))
-			 
-		      (t (tex-stripdollar (or (get x 'reversealias) x)))))
+			 ;; Do not apply stringdisp here -- we are outputting a string
+			 ;; only because we don't have a better way to handle Lisp arrays.
+			 (tex-string (quote-chars x "#$%&_"))))))
 	  r))
 
 (defun tex-string (x)
@@ -344,17 +344,45 @@
   (tex x (append l '("\\left(")) (cons "\\right)" r) 'mparen 'mparen))
 
 (defun tex-array (x l r)
+  (tex-array-display-indices x l r))
+
+(defun tex-array-display-indices (x l r)
+  (let*
+    ((base-symbol (caar x))
+     (indices (cdr x))
+     (display-indices (safe-mget base-symbol 'display-indices)))
+    (if (or (not display-indices) (not (= (length display-indices) (length indices))))
+      ;; Ignore DISPLAY-INDICES if it's empty, or nonempty and not the same size as INDICES.
+      (tex-array-simple x l r)
+      (let
+        ((pre-subscripts (extract-indices indices display-indices '$presubscript))
+         (pre-superscripts (extract-indices indices display-indices '$presuperscript))
+         (post-subscripts (extract-indices indices display-indices '$postsubscript))
+         (post-superscripts (extract-indices indices display-indices '$postsuperscript)))
+        (when (or pre-subscripts pre-superscripts)
+          (setq l (append l
+                          (list "{}")
+                          (if pre-subscripts (cons "_{" (tex-list pre-subscripts nil (list "}") ",")))
+                          (if pre-superscripts (cons "^{" (tex-list pre-superscripts nil (list "}") ","))))))
+        (when (or post-subscripts post-superscripts)
+          (setq r (append (if post-subscripts (cons "_{" (tex-list post-subscripts nil (list "}") ",")))
+                          (if post-superscripts  (cons "^{" (tex-list post-superscripts nil (list "}") ","))) r)))
+        (tex-atom base-symbol l r)))))
+
+(defun tex-array-simple (x l r)
   (let ((f))
+    ;; I believe this test always fails; TEX-MQAPPLY calls TEX-ARRAY w/ X = second argument of MQAPPLY.
     (if (eq 'mqapply (caar x))
-	(setq f (cadr x)
-	      x (cdr x)
-	      l (tex f (append l (list "\\left(")) (list "\\right)") 'mparen 'mparen))
-	(setq f (caar x)
-	      l (tex f l nil lop 'mfunction)))
+        (setq f (cadr x)
+              x (cdr x)
+              l (tex f (append l (list "\\left(")) (list "\\right)") 'mparen 'mparen))
+        (setq f (caar x)
+              l (tex f l nil lop 'mfunction)))
     (setq
      r (nconc (tex-list (cdr x) nil (list "}") ",") r))
     (nconc l (list "_{") r  )))
 
+;;
 ;; we could patch this so sin x rather than sin(x), but instead we made sin a prefix
 ;; operator
 
@@ -568,7 +596,7 @@
     (let ((r (exploden number)))
       (member 'e r :test #'string-equal))))
 
-(defvar *tex-mexpt-trig-like-fns* '(%sin %cos %tan %sinh %cosh %tanh %asin %acos %atan %asinh %acosh %atanh))
+(defvar *tex-mexpt-trig-like-fns* '(%sin %cos %tan %csc %sec %cot %sinh %cosh %tanh %asin %acos %atan %asinh %acosh %atanh))
 (defun tex-mexpt-trig-like-fn-p (f)
   (member f *tex-mexpt-trig-like-fns*))
 (defun maybe-tex-mexpt-trig-like (x l r)
@@ -585,8 +613,10 @@
        (doit (and
 	      f ; there is such a function
 	      (tex-mexpt-trig-like-fn-p f) ; f is trig-like
+          ;; I THINK THIS NEXT TEST IS UNNECESSARY BECAUSE IF IT PASSES THE PRECEDING TEST, IT IS ACCEPTABLE. REVISIT.
 	      (member (get-first-char f) '(#\% #\$) :test #'char=) ;; insist it is a % or $ function
 	      (not (member 'array (cdar fx) :test #'eq)) ; fix for x[i]^2
+          ;; I THINK THIS NEXT TEST IS UNNECESSARY BECAUSE NFORMAT CHANGES (...)^-1 TO 1/(...) AND (...)^(1/2) TO SQRT(...). REVISIT.
 	      (or (and (atom expon) (not (numberp expon))) ; f(x)^y is ok
 		  (and (atom expon) (numberp expon) (> expon 0))))))
                                         ; f(x)^3 is ok, but not f(x)^-1, which could
@@ -1150,7 +1180,9 @@
      ;; the Maxima function named by foo.
      (let ((s0 (nth 0 s)))
        (if (stringp s0)
-         (putprop e s0 'texword)
+         (progn
+           (when (get e 'texsym) (putprop e (list s0) 'texsym))
+           (putprop e s0 'texword))
          (make-maxima-tex-glue e s0)))) ;; assigns TEX property
 	((eq tx '$matchfix)
 	 (putprop e 'tex-matchfix 'tex)
@@ -1165,6 +1197,7 @@
 	((eq tx '$nofix)
 	 (putprop e 'tex-nofix 'tex)
 	 (putprop e s 'texsym)
+	 (when (get e 'texword) (putprop e (nth 0 s) 'texword))
 	 (car s))
 
 	((eq tx '$prefix)
@@ -1172,6 +1205,7 @@
 	 (when (null (get e 'grind))
 	   (putprop e 180 'tex-rbp))
 	 (putprop e s 'texsym)
+	 (when (get e 'texword) (putprop e (nth 0 s) 'texword))
 	 (car s))
 		
 	((eq tx '$infix)
@@ -1180,6 +1214,7 @@
 	   (putprop e 180 'tex-lbp)
 	   (putprop e 180 'tex-rbp))
 	 (putprop e  s 'texsym)
+	 (when (get e 'texword) (putprop e (nth 0 s) 'texword))
 	 (car s))
 
 	((eq tx '$nary)
@@ -1188,6 +1223,7 @@
 	   (putprop e 180 'tex-lbp)
 	   (putprop e 180 'tex-rbp))
 	 (putprop e s 'texsym)
+	 (when (get e 'texword) (putprop e (nth 0 s) 'texword))
 	 (car s))
 
 	((eq tx '$postfix)
@@ -1195,4 +1231,5 @@
 	 (when (null (get e 'grind))
 	   (putprop e 180 'tex-lbp))
 	 (putprop e  s 'texsym)
+	 (when (get e 'texword) (putprop e (nth 0 s) 'texword))
 	 (car s))))
