@@ -12,15 +12,9 @@
     (defmacro fb (op &rest args)
       `(,op ,@ (mapcar #'(lambda (x) `(the fixnum ,x)) args))))
 
-;; This function is not documented and not used in Maxima core or share.
-(defun $bt()
-  (loop for v in *baktrcl*
-	 do 
-	 (and (consp v)
-	      (consp (cadar v))
-	      (eq (caadar v) 'src))
-	 ($print (format nil "~a:~a:" (nth 1 (cadar v))
-			 (nth 0 (cadar v))) v)))
+(defstruct (line-info (:type list)) line file)
+
+(defstruct (bkpt (:type list)) form file file-line function)
 
 ;; *mlambda-call-stack*
 ;; #(NIL ($X) (1) $FF ($BIL $X) ($Y) (36) $JOE ($Y $BIL $X) ($JJX) (36)
@@ -31,7 +25,7 @@
 
 (defvar $mdebug_print_length 100 "Length of forms to print out in debugger")
 
-(defmacro bak-top-form (x) x)
+(defvar *lisp-quiet-suppressed-prompt* "" "The prompt lisp-quiet has suppressed")
 
 (defun frame-info (n)
   (declare (fixnum n))
@@ -49,16 +43,8 @@
     (setq bdlist (if (< m (fill-pointer ar)) (aref ar m) bindlist))
 					; (setq lineinfo (get-lineinfo backtr))
     (setq lineinfo (if ( < m (fill-pointer ar))
-		       (get-lineinfo (bak-top-form (aref ar (f+ m 1))))
-		       (get-lineinfo (bak-top-form *last-meval1-form*))))
-    ;;    #+if-you-use-baktrcl 
-    ;;	  (if ( < m (fill-pointer ar))
-    ;;	      (get-lineinfo (bak-top-form (aref ar (f+ m 1))))
-    ;;	    (or (get-lineinfo (bak-top-form *last-meval1-form*
-    ;;					    ;baktrcl
-    ;;					    ))
-    ;;		;(get-lineinfo (bak-top-form (cdr baktrcl)))
-    ;;		))
+		       (get-lineinfo (aref ar (f+ m 1)))
+		       (get-lineinfo *last-meval1-form*)))
     (values fname vals params backtr lineinfo bdlist)))
 
 (defun print-one-frame (n print-frame-number &aux val (st *debug-io*))
@@ -78,7 +64,7 @@
 			  (if (cdr v) "," "")))
 	   (princ ")" st)
 	   (and lineinfo
-		(format st (intl:gettext "(~a line ~a)")
+		(format st (intl:gettext " (~a line ~a)")
 			(short-name (cadr lineinfo)) (car lineinfo)))
 	   (terpri st)
 	   (values fname vals params backtr lineinfo bdlist))
@@ -86,60 +72,67 @@
 
 ;; these are in the system package in gcl...
 #-gcl
-(progn 'compile
-       (defun break-call (key args prop &aux fun)
-	 (setq fun (complete-prop key 'keyword prop))
-	 (setq key fun)
-	 (or fun (return-from break-call nil))
-	 (setq fun (get fun prop))
-	 (unless (symbolp fun)
-	   (let ((gen (gensym)))
-	     (setf (symbol-function gen) fun) (setf (get key prop) gen)
-	     (setq fun gen)))
-	 (cond (fun
-		(setq args (cons fun args))
+(progn
+  (defun break-call (key args prop &aux fun)
+    (setq fun (complete-prop key 'keyword prop))
+    (setq key fun)
+    (or fun (return-from break-call nil))
+    (setq fun (get fun prop))
+    (unless (symbolp fun)
+      (let ((gen (gensym)))
+	(setf (symbol-function gen) fun) (setf (get key prop) gen)
+	(setq fun gen)))
+    (cond (fun
+	   (setq args (cons fun args))
 					; jfa temporary hack
-		#+gcl(evalhook args nil nil *break-env*)
-		#-gcl(eval args)
-	        )
-	       (t (format *debug-io* 
-	                  (intl:gettext "~&~S is an undefined break command.~%")
-			  key))))
+	   #+gcl(evalhook args nil nil *break-env*)
+	   #-gcl(eval args)
+	   )
+	  (t (format *debug-io* 
+	             (intl:gettext "~&~S is an undefined break command.~%")
+		     key))))
 
-       (defun complete-prop (sym package prop &optional return-list)
-	 (cond ((and (symbolp sym)(get sym prop)(equal (symbol-package sym)
-						       (find-package package)))
-		(return-from complete-prop sym)))
-	 (loop for vv being the symbols of package 
-		when (and (get vv prop)
-			  (eql #+gcl (string-match sym vv)
-			       #-gcl (search (symbol-name sym)
-					     (symbol-name vv)) 
-			       0))
-		collect vv into all
-		finally
+  (defun complete-prop (sym package prop &optional return-list)
+    (cond ((and (symbolp sym)(get sym prop)(equal (symbol-package sym)
+						  (find-package package)))
+	   (return-from complete-prop sym)))
+    (loop for vv being the symbols of package 
+	  when (and (get vv prop)
+		    (eql #+gcl (string-match sym vv)
+			 #-gcl (search (symbol-name sym)
+				       (symbol-name vv)) 
+			 0))
+	    collect vv into all
+	  finally
        
-		(cond (return-list (return-from complete-prop all))
-		      ((> (length all) 1)
-               ;; NOTE TO TRANSLATORS: MEANING OF FOLLOWING IS UNKNOWN
-		       (format t
-		         (intl:gettext 
-                           "~&Not unique with property ~(~a: ~{~s~^, ~}~).")
-			       prop all))
-		      ((null all)
-		       (format t 
-		         (intl:gettext "~& No such break command: ~a") sym))
-		      (t (return-from complete-prop
-			   (car all)))))))
+	     (cond (return-list (return-from complete-prop all))
+		   ((> (length all) 1)
+		    (format *debug-io*
+			    (intl:gettext "~&Break command '~(~s~)' is ambiguous.~%")
+			    sym)
+		    (format *debug-io*
+			    (intl:gettext "Perhaps you meant one of the following: ~(~{~s~^, ~}~).")
+			    all)
+		    (finish-output *debug-io*))
+		   ((null all)
+		    (format *debug-io*
+			    (intl:gettext "~&Break command '~(~s~)' does not exist.")
+			    sym)
+		    (finish-output *debug-io*))
+		   (t (return-from complete-prop
+			(car all)))))))
 
-(defun $backtrace (&optional (n 30))
+(defmfun $backtrace (&optional (n 0 n-p))
+  (unless (typep n '(integer 0))
+    (merror
+      (intl:gettext "backtrace: number of frames must be a nonnegative integer; got ~M~%")
+      n))
   (let ($display2d)
-    (loop for i below n
-	   for j from *current-frame*
-	   while (print-one-frame j t))))
-
-;; the following are in the maxima package....
-;; they are DIFFERENT from ones in si package..
+    (loop for i from 0
+          for j from *current-frame*
+          when (and n-p (= i n))
+            return nil
+          while (print-one-frame j t))))
 
 ;; if this is NIL then nothing more is checked in eval
 
@@ -153,15 +146,13 @@
 (defvar *break-step* nil)
 (defvar *step-next* nil)
 
-(defun step-into (&optional (n 1))
-  ;;FORM is the next form about to be evaluated.
-  n
+(defun step-into (&optional ignored)
+  (declare (ignore ignored))
   (or *break-points* (init-break-points))
   (setq *break-step* 'break-step-into)
   :resume)
 
 (defun step-next (&optional (n 1))
-  n
   (let ((fun (current-step-fun)))
     (setq *step-next* (cons n fun))
     (or *break-points* (init-break-points))
@@ -179,7 +170,6 @@
 
 ;; These following functions, when they are the value of *break-step*
 ;; are invoked by an inner hook in eval. They may choose to stop things.
-(defvar *break-step* nil)
 (defun break-step-into (form &optional env)
   (let ((fun (current-step-fun)))
     (let ((line-info (set-full-lineinfo fun)))
@@ -266,12 +256,11 @@
 
 (declaim (special *mread-prompt*))
 
-;; RLT: What is the repeat-if-newline option for?  A grep of the code
-;; indicates that dbm-read is never called with more than 3 args.  Can
-;; we just flush it?  Can probably get rid of the &aux stuff too.
-
 (defvar *need-prompt* t)
 
+;; STREAM, EOF-ERROR-P and EOF-VALUE are analogous to the corresponding
+;; arguments to Lisp's READ.  REPEAT-IF-NEWLINE, when T, says to repeat
+;; the last break command (if available) when only a newline is read.
 (defun dbm-read (&optional (stream *standard-input*) (eof-error-p t)
 		 (eof-value nil) repeat-if-newline  &aux tem  ch
 		 (mprompt *mread-prompt*) (*mread-prompt* ""))
@@ -279,7 +268,7 @@
     (progn
       (fresh-line *standard-output*)
       (princ mprompt *standard-output*)
-      (force-output *standard-output*)
+      (finish-output *standard-output*)
       (setf *prompt-on-read-hang* nil))
     (progn
       (setf *prompt-on-read-hang* t)
@@ -335,8 +324,8 @@
 				(subseq 
 				 (read-line stream eof-error-p eof-value) 1))))
 		    `((displayinput) nil (($describe) ,line $exact))))
-         ((equal next #\?)
-          ;; Got "?? <stuff>". Invoke inexact search on <stuff>.
+		 ((equal next #\?)
+		  ;; Got "?? <stuff>". Invoke inexact search on <stuff>.
 		  (let* ((line (string-trim 
 				'(#\space #\tab #\; #\$)
 				(subseq 
@@ -371,17 +360,15 @@
 (defvar *break-env* nil)
 (defvar *top-eof* (cons nil nil))
 (defvar *quit-tag* 'macsyma-quit)
-;; should maybe be??
-;;(defvar *quit-tag* 'si::*quit-tag*)
 
 (defvar *quit-tags* nil)
 
 (defun set-env (bkpt)
   (format *debug-io* 
-          (intl:gettext "(~a ~a~@[ in ~a~])") 
+          (intl:gettext "(~a line ~a~@[, in function ~a~])")
           (short-name (bkpt-file bkpt))
 	  (bkpt-file-line bkpt)
-	  nil)				; (bkpt-function bkpt)
+	  (bkpt-function bkpt))
   (format *debug-io* "~&~a:~a::~%" (bkpt-file bkpt)
 	  (bkpt-file-line bkpt)))
 
@@ -405,8 +392,8 @@
       (catch *quit-tag*
 	(unwind-protect
 	     (do () (())
-	       (format-prompt *debug-io* "~a"
-			      (format nil "~&~@[(~a:~a) ~]"
+	       (format-prompt *debug-io* "~&~a"
+			      (format nil "~@[(~a:~a) ~]"
 				      (unless (stringp at) "dbm")
 				      (length *quit-tags*)))
 	       (finish-output *debug-io*)
@@ -458,9 +445,10 @@
 	     (dolist (v (complete-prop key 'keyword 'break-doc t))
 	       (format t "~&~%~(~s~)   ~a" v (get v 'break-doc)))))
 	(t
+	 ; Skip any undocumented break commands
 	 (loop for vv being the symbols of 'keyword
-		when (get vv 'break-command)
-		collect (cons vv (or (get vv 'break-doc) "Undocumented"))
+		when (and (get vv 'break-command) (get vv 'break-doc))
+		collect (cons vv (get vv 'break-doc))
 		into all
 		finally (setq all (sort all 'alphalessp))
 	      (format t (intl:gettext "~
@@ -477,7 +465,9 @@ Command      Description~%~
   "Print help on a break command or with no arguments on
              all break commands")
 
-;; What is this debug command for?
+;; This is an undocumented break command which gets placed in
+;; *LAST-DBM-COMMAND* when an invalid (nonexistent or ambiguous)
+;; break command is read in.
 (def-break :_none #'(lambda()) nil)
 
 (def-break :next  'step-next
@@ -494,13 +484,9 @@ Command      Description~%~
 (def-break :top  #'(lambda( &rest l)l (throw 'macsyma-quit 'top)) 
   "Throw to top level")
 
-(defstruct (line-info (:type list)) line file)
-
-(defstruct (bkpt (:type list)) form file file-line function)
-
 (defun *break-points* (form)
   (let ((pos(position form *break-points* :key 'car)))
-    (format t "Bkpt ~a:" pos)
+    (format *debug-io* "Bkpt ~a: " pos)
     (break-dbm-loop  (aref *break-points* pos))))
 
 ;; fun = function name eg '$|odeSeriesSolve| and 
@@ -510,7 +496,7 @@ Command      Description~%~
 (defun break-function (fun &optional (li 0) absolute 
                            &aux i tem info form fun-line)
   (unless *mdebug*
-    (format t "~&Turning on debugging debugmode(true)")
+    (format t "~&Turning on debugging debugmode(true)~%")
     (setq *mdebug* t))
   (cond ((stringp fun)
 	 (let ((file fun)  start)
@@ -592,11 +578,11 @@ Command      Description~%~
       (when (eq (car bpt) nil)
 	(setq disabled t)
 	(setq bpt (cdr bpt)))
-      (format t "Bkpt ~a:(~a line ~a)~@[(disabled)~]"
+      (format t "Bkpt ~a: (~a line ~a)~@[ (disabled)~]"
 	      n (short-name (second bpt))
 	      (third bpt) disabled)
       (let ((fun (fourth bpt)))
-	(format t "(line ~a of ~a)"  (relative-line fun (nth 2 bpt))
+	(format t " (line ~a of ~a)" (relative-line fun (nth 2 bpt))
 		fun)))))
 
 (defun relative-line (fun l)
@@ -627,7 +613,8 @@ Command      Description~%~
 		       tem))
 		  (:show
 		   (when tem (show-break-point i)
-			 (terpri))
+			 (terpri)
+			 (finish-output))
 		   tem)))))))
 
 ;; get the most recent function on the stack with step info.
@@ -649,6 +636,8 @@ Command      Description~%~
   "Print information about item")
 
 (defmacro lisp-quiet (&rest l)
+  (if (not (string= *mread-prompt* ""))
+      (setq *lisp-quiet-suppressed-prompt* *mread-prompt*))
   (setq *mread-prompt* "")
   (eval (cons 'progn l))
   nil)
@@ -660,6 +649,9 @@ Command      Description~%~
   "Evaluate the lisp form following on the line")
 
 (defmacro lisp-eval (&rest l)
+  (if (string= *mread-prompt* "")
+      (setq *mread-prompt* *lisp-quiet-suppressed-prompt*))
+  
   (dolist (v (multiple-value-list (eval (cons 'progn l))))
     (fresh-line *standard-output*)
     (princ v)))
