@@ -21,14 +21,14 @@
 	  (mfunction1 (setq bfun (getopr f))))
     (when (member bfun '(lessthan great) :test #'eq)
       (setq llist (mapcar #'ratdisrep llist)))
-    (cons '(mlist) (sort llist comparfun))))
+    (cons '(mlist) (stable-sort llist comparfun))))
 
 ;; cmulisp does not like the closure version.  Clisp insists on the
 ;; closure version.  Gcl likes either...  For the moment we will
 ;; leave a conditional here.
 (defun mfunction1 (fun)
   (if (functionp fun)
-      fun
+      (lambda (x y) (mevalp (funcall fun x y)))
       #+(or cmu scl)
       (lambda (x y) (mevalp `((,fun) ((mquote) ,x) ((mquote) ,y))))
       #-(or cmu scl)
