@@ -1,5 +1,13 @@
-;;; Compiled by f2cl version 2.0 beta Date: 2007/05/04 17:29:50 
-;;; Using Lisp CMU Common Lisp Snapshot 2007-05 (19D)
+;;; Compiled by f2cl version:
+;;; ("f2cl1.l,v 46c1f6a93b0d 2012/05/03 04:40:28 toy $"
+;;;  "f2cl2.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl3.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl4.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl5.l,v 46c1f6a93b0d 2012/05/03 04:40:28 toy $"
+;;;  "f2cl6.l,v 1d5cbacbb977 2008/08/24 00:56:27 rtoy $"
+;;;  "macros.l,v fceac530ef0c 2011/11/26 04:02:26 toy $")
+
+;;; Using Lisp CMU Common Lisp snapshot-2012-04 (20C Unicode)
 ;;; 
 ;;; Options: ((:prune-labels nil) (:auto-save t) (:relaxed-array-decls t)
 ;;;           (:coerce-assigns :as-needed) (:array-type ':simple-array)
@@ -410,15 +418,15 @@
                                        -4.970947029753337e-32)))
       (pi4 0.7853981633974483)
       (first$ nil))
-  (declare (type f2cl-lib:logical first$)
-           (type (simple-array double-float (72)) ath2cs)
-           (type (simple-array double-float (74)) am22cs)
-           (type (simple-array double-float (58)) ath1cs)
-           (type (simple-array double-float (60)) am21cs)
-           (type (simple-array double-float (53)) ath0cs)
+  (declare (type (f2cl-lib:integer4) nam20 nath0 nam21 nath1 nam22 nath2)
+           (type (double-float) xsml pi4)
            (type (simple-array double-float (57)) am20cs)
-           (type (double-float) pi4 xsml)
-           (type (integer) nath2 nam22 nath1 nam21 nath0 nam20))
+           (type (simple-array double-float (53)) ath0cs)
+           (type (simple-array double-float (60)) am21cs)
+           (type (simple-array double-float (58)) ath1cs)
+           (type (simple-array double-float (74)) am22cs)
+           (type (simple-array double-float (72)) ath2cs)
+           (type f2cl-lib:logical first$))
   (setq first$ f2cl-lib:%true%)
   (defun d9aimp (x ampl theta)
     (declare (type (double-float) theta ampl x))
