@@ -40,7 +40,7 @@
 		     finally (return t)))
 	 ;;(EVERY #'ATOM (CDR EXP)))
 	 exp1)
-	(t (merror (intl:gettext "UNIPOLY: argument must be a univariate polynomial; found: ~M") exp))))
+	(t (merror (intl:gettext "UNIPOLY: argument must be a univariate polynomial with rational coefficients; found: ~M") exp))))
 
 (defun makrat (pt)
   (cond ((floatp pt) (maxima-rationalize pt))
@@ -64,7 +64,7 @@
 
 (declare-top (unspecial equations))
 
-(defmfun sturm1 (poly eps &aux b llist)
+(defun sturm1 (poly eps &aux b llist)
   (setq b (cons (root-bound (cdr poly)) 1))
   (setq llist (isolat poly (cons (- (car b)) (cdr b)) b))
   (mapcar #'(lambda (int) (refine poly (car int) (cdr int) eps)) llist))
