@@ -34,6 +34,7 @@
   #+clisp   (ext:getenv envvar)
   #+(or openmcl mcl)     (ccl::getenv envvar)
   #+lispworks (hcl:getenv envvar)
+  #+abcl (ext:getenv envvar)
   )
 
 ;; CMUCL needs because when maxima reaches EOF, it calls BYE, not $QUIT.
@@ -41,11 +42,13 @@
 (defun bye ()
   #+(or cmu scl clisp) (ext:quit)
   #+sbcl               (sb-ext:quit)
-  #+allegro            (excl:exit)
+  #+allegro            (excl:exit 0 :quiet t)
   #+(or mcl openmcl)   (ccl:quit)
-  #+gcl                (lisp:quit)
+  #+gcl                (system::quit)
   #+ecl                (si:quit)
   #+lispworks          (lispworks:quit)
+  #+abcl               (cl-user::quit)
+  #+kcl                (lisp::bye)
   )
 
 
@@ -53,7 +56,7 @@
 ;;; and applied to consequtive pairs of elements of the list.
 ;;; Useful for iterating over property lists.
 
-(defmfun map2c (f l)
+(defun map2c (f l)
   (do ((llt l (cddr llt)) (lans))
       ((null llt) lans)
     (push (funcall f (car llt) (cadr llt)) lans)))
@@ -61,7 +64,7 @@
 ;;; Like MAPCAR, except if an application of F to any of the elements of L
 ;;; returns NIL, then the function returns NIL immediately.
 
-(defmfun andmapcar (f l &aux d answer)
+(defun andmapcar (f l &aux d answer)
   (do ((l l (cdr l)))
       ((null l) (nreverse answer))
     (setq d (funcall f (car l)))
@@ -69,7 +72,7 @@
 
 ;;; Returns T if either A or B is NIL, but not both.
 
-(defmfun xor (a b)
+(defun xor (a b)
   (or (and (not a) b) (and (not b) a)))
   
 ;;; A MEMQ which works at all levels of a piece of list structure.
@@ -77,7 +80,7 @@
 ;;; Note that (AMONG NIL '(A B C)) is T, however.  This could cause bugs.
 ;;; > This is false. (AMONG NIL anything) returns NIL always. -kmp
 
-(defmfun among (x l) 
+(defun among (x l) 
   (cond ((null l) nil)
 	((atom l) (eq x l))
 	(t (or (among x (car l)) (among x (cdr l)))))) 
@@ -85,16 +88,27 @@
 ;;; Similar to AMONG, but takes a list of objects to look for.  If any
 ;;; are found in L, returns T.
 
-(defmfun amongl (x l) 
+(defun amongl (x l) 
   (cond ((null l) nil)
 	((atom l) (member l x :test #'eq))
 	(t (or (amongl x (car l)) (amongl x (cdr l)))))) 
 
+;;; Tests to see whether one tree is a subtree of another.
+;;;
+;;; Both arguments should be well-formed cons trees (so no cycles). If supplied,
+;;; TEST is used as an equality predicate.
+
+(defun subtree-p (branch tree &key (test #'eql))
+  (or (funcall test branch tree)
+      (and (not (atom tree))
+           (member branch tree
+                   :test (lambda (x y) (subtree-p x y :test test))))))
+
 ;;; Takes a list in "alist" form and converts it to one in
 ;;; "property list" form, i.e. ((A . B) (C . D)) --> (A B C D).
 ;;; All elements of the list better be conses.
 
-(defmfun dot2l (l)
+(defun dot2l (l)
   (cond ((null l) nil)
 	(t (list* (caar l) (cdar l) (dot2l (cdr l))))))
 
@@ -107,7 +121,7 @@
 ;;;     else store the value under the appropriate property.
 ;;;
 
-(defmfun cput (bas val sel)
+(defun cput (bas val sel)
   (cond ((null val)
 	 (zl-remprop bas sel)
 	 nil)
